@@ -32,7 +32,6 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.11 2006/03/27 23:24:50 abe Exp $";
 #endif
 
 
@@ -83,7 +82,7 @@ getvpath(va, rv)
 	    if (!(bp = (char *)malloc((MALLOC_S)bl))) {
 		(void) fprintf(stderr, "%s: no space (%d) for path assembly\n",
 		    Pn, (int)bl);
-		Exit(1);
+		Error();
 	    }
 	}
 	pp = bp + bl - 1;
@@ -221,7 +220,7 @@ getvpath(va, rv)
 		if (!(cb = (char *)malloc((MALLOC_S)(MAXPATHLEN + 1)))) {
 		    (void) fprintf(stderr, "%s: no space (%d) for CWD\n",
 			Pn, (int)bl);
-		    Exit(1);
+		    Error();
 		}
 		if (!getcwd(cb, (size_t)(MAXPATHLEN + 1))) {
 		    if (!Fwarn) {
@@ -286,7 +285,7 @@ getvpath_alloc:
 	if (!(ap = (char *)malloc(pl + 1))) {
 	    (void) fprintf(stderr, "%s: no getvpath space (%d)\n",
 		Pn, pl + 1);
-	    Exit(1);
+	    Error();
 	}
 	(void) memmove(ap, pp, pl + 1);
 	return(ap);
@@ -856,7 +855,7 @@ process_node(va)
 	     * Save node number.
 	     */
 		Lf->inode = (INODETYPE)sb.st_ino;
-		Lf->inp_ty = 1; 
+		Lf->inp_ty = 1;
 	    /*
 	     * Optionally save link count.
 	     */
