@@ -32,7 +32,6 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.17 2001/01/09 15:00:07 abe Exp $";
 #endif
 
 
@@ -258,7 +257,8 @@ isvlocked(vp)
 	    if (lp->set.l_rsys || lp->set.l_pid != (pid_t)Lp->pid)
 		continue;
 	    if (lp->set.l_whence == 0 && lp->set.l_start == 0
-	    &&  lp->set.l_len == 0x7fffffffffffffff)
+	    &&  ((lp->set.l_len == 0x8000000000000000)
+	    ||   (lp->set.l_len == 0x7fffffffffffffff)))
 		l = 1;
 	    else
 		l = 0;
@@ -305,7 +305,7 @@ load_flinfo()
 		(void) fprintf(stderr,
 		    "%s: can't allocate %d byte local lock hash buckets\n",
 		    Pn, L_FLINFO_HSZ * sizeof(struct l_flinfo *));
-		Exit(1);
+		Error();
 	    }
 	}
 /*
@@ -328,7 +328,7 @@ load_flinfo()
 	    if (!(lfi = (struct l_flinfo *)malloc(sizeof(struct l_flinfo)))) {
 		(void) fprintf(stderr,
 		    "%s: no space for local vnode lock info struct\n", Pn);
-		Exit(1);
+		Error();
 	    }
 	    lfi->vp = fi.vp;
 	    lfi->lp = (struct l_lock *)NULL;
@@ -347,7 +347,7 @@ load_flinfo()
 		if (!(ll = (struct l_lock *)malloc(sizeof(struct l_lock)))) {
 		    (void) fprintf(stderr,
 			"%s: no space for local lock struct\n", Pn);
-		    Exit(1);
+		    Error();
 		}
 		ll->next = lfi->lp;
 		lfi->lp = ll;
@@ -418,13 +418,13 @@ process_node(va)
 	 */
 	    if (!(v = (struct vnode *)malloc(sizeof(struct vnode)-1+Vnmxp))) {
 		(void) fprintf(stderr, "%s: no space for vnode buffer\n", Pn);
-		Exit(1);
+		Error();
 	    }
 
 #if	DUV>=30000
 	    if (!(fv = (struct vnode *)malloc(sizeof(struct vnode)-1+Vnmxp))) {
 		(void) fprintf(stderr, "%s: no space for fvnode buffer\n", Pn);
-		Exit(1);
+		Error();
 	    }
 #endif	/* DUV>=30000 */
 
@@ -436,9 +436,9 @@ process_node(va)
 
 #if	defined(HASNCACHE)
 	Lf->na = va;
-# if	defined(HASNCAPID)
+# if	defined(HASNCVPID)
 	Lf->id = (unsigned long)v->v_id;
-# endif	/* defined(HASNCAPID) */
+# endif	/* defined(HASNCVPID) */
 #endif	/* defined(HASNCACHE) */
 
 #if	defined(HASFSTRUCT)
@@ -607,6 +607,10 @@ process_node(va)
 	} else if (r) {
 	    dev = r->r_attr.va_fsid;
 	    devs = 1;
+	    if ((type == VCHR) || (type == VBLK)) {
+		rdev = r->r_attr.va_rdev;
+		rdevs = 1;
+	    }
 	} else if (s) {
 	    if (vfs && vfs->dev)
 		dev = vfs->dev;
@@ -616,7 +620,7 @@ process_node(va)
 	    rdev = s->sn_vattr.va_rdev;
 	    rdevs = 1;
 	    if (!lkupdev(&dev, &rdev, 0, 0) && HaveCloneMaj)
-		rdev = makedev(CloneMaj, major(rdev));
+		rdev = makedev(CloneMaj, GET_MAJ_DEV(rdev));
 	} else if (s5) {
 	    dev = s5->i_dev;
 	    devs = 1;
@@ -633,11 +637,11 @@ process_node(va)
 
 #if	ADVFSV>=500
 	    if (fscs) {
-		Lf->inode = (unsigned long)fsc.st.num;
+		Lf->inode = (INODETYPE)fsc.st.num;
 		Lf->inp_ty = 1;
 	    }
 #else	/* ADVFSV<500 */
-	    Lf->inode = (unsigned long)a->a_number;
+	    Lf->inode = (INODETYPE)a->a_number;
 	    Lf->inp_ty = 1;
 #endif	/* ADVFSV>=500 */
 
@@ -652,37 +656,37 @@ process_node(va)
 #endif	/* defined(HASTAGTOPATH) */
 
 	} else if (c) {
-	    Lf->inode = (unsigned long)c->cd_number;
+	    Lf->inode = (INODETYPE)c->cd_number;
 	    Lf->inp_ty = 1;
 	}
 
 #if	DUV>=50000
 	else if (cn) {
 	    if (cn->c_attr.va_mask & AT_NODEID) {
-		Lf->inode = (unsigned long)cn->c_attr.va_fileid;
+		Lf->inode = (INODETYPE)cn->c_attr.va_fileid;
 		Lf->inp_ty = 1;
 	    }
 	}
 #endif	/* DUV>=50000 */
 
 	else if (i) {
-	    Lf->inode = (unsigned long)i->i_number;
+	    Lf->inode = (INODETYPE)i->i_number;
 	    Lf->inp_ty = 1;
 	} else if (p) {
-	    Lf->inode = (unsigned long)((type == VDIR) ? PR_ROOTINO
+	    Lf->inode = (INODETYPE)((type == VDIR) ? PR_ROOTINO
 					: p->prc_pid + PR_INOBIAS);
 	    Lf->inp_ty = 1;
 	} else if (r) {
-	    Lf->inode = (unsigned long)r->r_attr.va_fileid;
+	    Lf->inode = (INODETYPE)r->r_attr.va_fileid;
 	    Lf->inp_ty = 1;
 	} else if (s5) {
-	    Lf->inode = (unsigned long)s5->i_number;
+	    Lf->inode = (INODETYPE)s5->i_number;
 	    Lf->inp_ty = 1;
 	}
 
 #if	DUV>=30000
 	else if (f) {
-	    Lf->inode = (unsigned long)f->fn_fileid;
+	    Lf->inode = (INODETYPE)f->fn_fileid;
 	    Lf->inp_ty = 1;
 	}
 #endif	/* DUV>=30000 */
@@ -762,7 +766,7 @@ process_node(va)
 			Lf->sz = (SZOFFTYPE)s5->i_size;
 			Lf->sz_def = 1;
 		    }
-		} else if (type == VCHR && !Fsize)
+		} else if ((type == VBLK) || (type == VCHR) && !Fsize)
 		    Lf->off_def = 1;
 	    }
 	}
@@ -842,6 +846,7 @@ process_node(va)
 	    break;
 	case VBLK:
 	    ty = "VBLK";
+	    Ntype = N_BLK;
 	    break;
 	case VCHR:
 	    ty = "VCHR";
@@ -874,11 +879,7 @@ process_node(va)
 
 	    break;
 	default:
-	    if (type > 9999)
-		(void) snpf(Lf->type, sizeof(Lf->type), "*%03d", type % 1000);
-	    else
-		(void) snpf(Lf->type, sizeof(Lf->type), "%4d", type);
-	    (void) snpf(Namech, Namechl, "unknown type");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "%04o", (type & 0xfff));
 	    ty = (char *)NULL;
 	}
 	if (ty)
