@@ -31,13 +31,14 @@
 
 
 /*
- * $Id: dlsof.h,v 1.47 2015/07/07 20:23:43 abe Exp $
+ * $Id: dlsof.h,v 1.49 2018/07/14 12:14:55 abe Exp $
  */
 
 
 #if	!defined(FREEBSD_LSOF_H)
 #define	FREEBSD_LSOF_H	1
 
+#include <stddef.h>
 #include <stdlib.h>
 #include <dirent.h>
 #include <nlist.h>
@@ -45,8 +46,12 @@
 #include <signal.h>
 #include <unistd.h>
 
-# if	FREEBSDV>=4000
-#  if	FREEBSDV>=5000
+#if    FREEBSDV>=13000
+/* This header is a huge mess.  Please don't support EOL FreeBSD releases. */
+#define        _KERNEL 1
+#include <sys/_lock.h>
+#undef _KERNEL
+#endif         /* FREEBSDV>=13000 */
 #   if	FREEBSDV<6020
 #    if	defined(__alpha__)
 /*
@@ -55,7 +60,7 @@
  * complaint about register use.
  */
 
-#include <machine/pcpu.h>       
+#include <machine/pcpu.h>
 #define PCPU_MD_FIELDS                                                  \
 	struct alpha_pcb pc_idlepcb;            /* pcb for idling */    \
 	u_int64_t       pc_idlepcbphys;         /* pa of pc_idlepcb */  \
@@ -65,7 +70,6 @@
 #    endif	/* defined(__alpha__) */
 #   endif	/* FREEBSDV<6020 */
 #define	_KERNEL	1
-#  endif	/* FREEBSDV>=5000 */
 
 #  if	defined(HAS_VM_MEMATTR_T)
 /*
@@ -87,6 +91,26 @@
 #define	boolean_t	int
 #  endif	/* defined(NEEDS_BOOLEAN_T) */
 
+#  if defined(NEEDS_DEVICE_T)
+/*
+ * In FreeBSD 12 <sys/conf.h calls <sys/eventhandler.h> and it needs
+ * the device_t typedef.
+ */
+typedef	struct device	*device_t;
+#endif /* defined(NEEDS_DEVICE_T) */
+
+/*
+ * Define KLD_MODULE to avoid the error "ARM_NMMUS is 0" from ARM's
+ * <machine/cpuconf.h>.
+ */
+
+#define KLD_MODULE
+
+/*
+ * include <stdbool.h> for refount(9)
+ */
+#include <stdbool.h>
+
 #include <sys/conf.h>
 
 #  if	defined(HAS_VM_MEMATTR_T)
@@ -102,23 +126,17 @@
 #include "fbsd_minor.h"
 #  endif	/* defined(HAS_CONF_MINOR) */
 
-#  if	FREEBSDV>=5000
 #undef	_KERNEL
-#  endif	/* FREEBSDV>=5000 */
-# endif	/* FREEBSDV>=4000 */
 
 #include <sys/filedesc.h>
 #include <sys/mbuf.h>
 #define	NFS
 #define m_stat	mnt_stat
 
-# if	FREEBSDV>=3020
 #define	_KERNEL
-# endif	/* FREEBSDV>=3020 */
 
 #include <sys/mount.h>
 
-# if	FREEBSDV>=3020
 #  if	defined(__clang__)
 /*
  * This definition is needed when clang is used, because <sys/mount.h> must
@@ -129,29 +147,30 @@ int     getmntinfo(struct statfs **, int);
 #  endif	/* defined(__clang__) */
 
 #undef	_KERNEL
-# endif	/* FREEBSDV>=3020 */
 
 #include <rpc/types.h>
 #include <sys/protosw.h>
 #include <sys/socket.h>
+#define	_WANT_SOCKET
 #include <sys/socketvar.h>
 #include <sys/un.h>
+#define	_WANT_UNPCB
 #include <sys/unpcb.h>
 
-# if	FREEBSDV>=3000
 #undef	INADDR_LOOPBACK
-# endif	/* FREEBSDV>=3000 */
 
 #include <netinet/in.h>
 #include <netinet/in_systm.h>
 #include <netinet/ip.h>
 #include <net/route.h>
+#define	_WANT_INPCB			/* for FreeBSD 12 and above */
 #include <netinet/in_pcb.h>
 #include <netinet/ip_var.h>
 #include <netinet/tcp.h>
 #include <netinet/tcpip.h>
 #include <netinet/tcp_fsm.h>
 #include <netinet/tcp_timer.h>
+#define	_WANT_TCPCB			/* for FreeBSD 12 and above */
 #include <netinet/tcp_var.h>
 #include <sys/ucred.h>
 #include <sys/uio.h>
@@ -171,21 +190,9 @@ int     getmntinfo(struct statfs **, int);
 #include <rpc/pmap_prot.h>
 #undef	pmap
 
-# if	FREEBSDV<2000
-#include <ufs/quota.h>
-#include <ufs/inode.h>
-#include <ufs/ufsmount.h>
-#include <ufs/mfsnode.h>
-# else	/* FREEBSDV>=2000 */
 #include <paths.h>
 #include <ufs/ufs/quota.h>
 
-#  if	FREEBSDV>=4000 && FREEBSDV<5000
-#   if	defined(__alpha__) || defined(__sparc64__)
-#define	dev_t	void *
-#   endif	/* defined(__alpha__) || defined(__sparc64__) */
-#  endif /* FREEBSDV>=4000 && FREEBSDV<5000 */
-
 #include <ufs/ufs/inode.h>
 
 # if	defined(HAS_UFS1_2)
@@ -219,59 +226,28 @@ struct vop_setextattr_args;
 #undef	_KERNEL
 # endif	/* defined(HAS_UFS1_2) */
 
-#  if	FREEBSDV>=5010
 #undef	i_devvp
-#  endif	/* FREEBSDV>=5010 */
 
-#  if	FREEBSDV>=4000 && FREEBSDV<5000
-#   if	defined(__alpha__) || defined(__sparc64__)
-#undef	dev_t
-#   endif	/* defined(__alpha__) || defined(__sparc64__) */
-#  endif /* FREEBSDV>=4000 && FREEBSDV<5000 */
-
-#  if   FREEBSDV<2020
-#include <ufs/mfs/mfsnode.h>
-#  endif        /* FREEBSDV<2020 */
-
-# endif	/* FREEBSDV<2000 */
-
-# if	FREEBSDV<5000
-#include <nfs/nfsv2.h>
-# else	/* FREEBSDV>=5000 */
 #include <nfs/nfsproto.h>
-# endif	/* FREEBSDV<5000 */
 
 # if	defined(HASRPCV2H)
 #include <nfs/rpcv2.h>
 # endif	/* defined(HASRPCV2H) */
 
-# if	FREEBSDV>=5000
 #include <nfsclient/nfs.h>
 #include <nfsclient/nfsnode.h>
-# else	/* FREEBSDV<5000 */
-#include <nfs/nfs.h>
-#include <nfs/nfsnode.h>
-# endif	/* FREEBSDV>=5000 */
 
 #include <sys/proc.h>
 #include <kvm.h>
 #undef	TRUE
 #undef	FALSE
 
-# if	FREEBSDV<2000
-#include <sys/kinfo.h>
-# else	/* FREEBSDV>=2000 */
 #include <sys/sysctl.h>
-# endif	/* FREEBSDV<2000 */
 
 # if	defined(HASFDESCFS)
 #define	_KERNEL
 #define	KERNEL
-#  if	FREEBSDV>=5000
 #include <fs/fdescfs/fdesc.h>
-#  else	/* FREEBSDV<5000 */
-#include <miscfs/fdesc/fdesc.h>
-#  endif	/* FREEBSDV>=5000 */
 #undef	_KERNEL
 #undef	KERNEL
 # endif	/* defined(HASFDESCFS) */
@@ -280,24 +256,13 @@ struct vop_setextattr_args;
 #define	_KERNEL
 #define	KERNEL
 struct vop_generic_args;
-#  if	FREEBSDV>=5000
 #include <fs/nullfs/null.h>
-#  else	/* FREEBSDV<5000 */
-#include <miscfs/nullfs/null.h>
-#  endif	/* FREEBSDV>=5000 */
 #undef	_KERNEL
 #undef	KERNEL
 # endif	/* defined(HASNULLFS) */
 
 # if	defined(HASPROCFS)
-#  if	FREEBSDV<2000
-#include <procfs/pfsnode.h>
-# else	/* FREEBSDV>=2000 */
-#  if	FREEBSDV<5000
-#include <miscfs/procfs/procfs.h>
-#  endif	/* FREEBSDV<5000 */
 #include <machine/reg.h>
-# endif	/* FREEBSDV<2000 */
 
 #define	PNSIZ		5
 # endif	/* defined(HASPROCFS) */
@@ -311,24 +276,6 @@ struct vop_generic_args;
 # endif	/* defined(HAS_ZFS) */
 
 
-# if	FREEBSDV<2000
-#define	P_COMM		p_comm
-#define	P_FD		p_fd
-#define	P_PID		p_pid
-#define	P_PGID		p_pgrp
-#define	P_STAT		p_stat
-#define	P_VMSPACE	p_vmspace
-# else	/* FREEBSDV>=2000 */
-#  if	FREEBSDV<5000
-#define	P_ADDR		kp_eproc.e_paddr
-#define	P_COMM		kp_proc.p_comm
-#define	P_FD		kp_proc.p_fd
-#define	P_PID		kp_proc.p_pid
-#define	P_PGID		kp_eproc.e_pgid
-#define	P_PPID		kp_eproc.e_ppid
-#define	P_STAT		kp_proc.p_stat
-#define	P_VMSPACE	kp_proc.p_vmspace
-#  else	/* FREEBSDV>=5000 */
 #define	P_ADDR		ki_paddr
 #define	P_COMM		ki_comm
 #define	P_FD		ki_fd
@@ -337,8 +284,6 @@ struct vop_generic_args;
 #define	P_PPID		ki_ppid
 #define	P_STAT		ki_stat
 #define	P_VMSPACE	ki_vmspace
-#  endif	/* FREEBSDV<5000 */
-# endif	/* FREEBSDV<2000 */
 
 #include <vm/vm.h>
 
@@ -400,20 +345,12 @@ int     open(const char *, int, ...);
 
 # if	defined(DTYPE_KQUEUE)
 #define	HASKQUEUE				/* has the kqueue file type */
-#   if	FREEBSDV>=4090
 #define	_KERNEL
-#   endif	/* FREEBSDV>=4090 */
 #include <sys/eventvar.h>
-#   if	FREEBSDV>=4090
 #undef	_KERNEL
-#   endif	/* FREEBSDV>=4090 */
 # endif	/* defined(DTYPE_KQUEUE) */
 
-# if	FREEBSDV<2000
-#include <ufs/lockf.h>
-# else	/* FREEBSDV>=2000 */
 struct vop_advlock_args { int dummy; };	/* to pacify lf_advlock() prototype */
-#  if	FREEBSDV>=5000
 #undef	MALLOC_DECLARE
 #define	MALLOC_DECLARE(type)	extern struct malloc_type type[1]
 					/* to pacify <sys/lockf.h> */
@@ -429,23 +366,15 @@ struct vop_advlock_args { int dummy; };	/* to pacify lf_advlock() prototype */
 
 #include <fs/devfs/devfs.h>
 #undef	_KERNEL
-#  endif	/* FREEBSDV>=5000 */
 #include <sys/lockf.h>
-# endif	/* FREEBSDV<2000 */
 
-#  if   FREEBSDV>=2020
-#   if	FREEBSDV>=4090
 #define	_KERNEL
-#   endif	/* FREEBSDV>=4090 */
 #include <sys/pipe.h>
-#   if	FREEBSDV>=4090
 #undef	_KERNEL
-#   endif	/* FREEBSDV>=4090 */
 #   if	defined(HASVMLOCKH)
 #include <vm/lock.h>
 #   endif	/* defined(HASVMLOCKH) */
 #include <vm/pmap.h>
-#  endif        /* FREEBSDV>=2020 */
 
 #include <vm/vm_map.h>
 
@@ -461,21 +390,20 @@ struct vop_advlock_args { int dummy; };	/* to pacify lf_advlock() prototype */
 #include <vm/vm_object.h>
 #include <vm/vm_pager.h>
 
-# if   FREEBSDV>=2020
 #undef	B_NEEDCOMMIT
 
-#  if	FREEBSDV>=5000
 #include <sys/bio.h>
-#  endif	/* FREEBSDV>=5000 */
 
 #include <sys/buf.h>
 #include <sys/user.h>
 
-#  if	FREEBSDV<5000
-#include <ufs/mfs/mfsnode.h>
-#  endif	/* FREEBSDV<5000 */
-# endif        /* FREEBSDV>=2020 */
-
+#undef	bcmp		/* avoid _KERNEL conflict */
+#undef	bcopy		/* avoid _KERNEL conflict */
+#undef	bzero		/* avoid _KERNEL conflict */
+#undef	memcmp		/* avoid _KERNEL conflict */
+#undef	memcpy		/* avoid _KERNEL conflict */
+#undef	memmove		/* avoid _KERNEL conflict */
+#undef	memset		/* avoid _KERNEL conflict */
 #include <string.h>
 
 
@@ -483,11 +411,7 @@ struct vop_advlock_args { int dummy; };	/* to pacify lf_advlock() prototype */
 #define DEVINCR		1024	/* device table malloc() increment */
 
 # if	!defined(FREEBSD_KA_T)
-#  if	FREEBSDV<2000
-typedef	off_t		KA_T;
-#  else	/* FREEBSDV>=2000 */
 typedef	u_long		KA_T;
-#  endif	/* FREEBSDV<2000 */
 # endif	/* !defined(FREEBSD_KA_T) */
 
 #define	KMEM		"/dev/kmem"
@@ -521,11 +445,9 @@ typedef	u_long		KA_T;
  * Global storage definitions (including their structure definitions)
  */
 
-struct file * Cfp;
+extern struct file * Cfp;
 
-# if	FREEBSDV>=2000
 extern kvm_t *Kd;
-# endif	/* FREEBSDV>=2000 */
 
 # if	defined(P_ADDR)
 extern KA_T Kpa;
@@ -561,15 +483,15 @@ struct mounts {
         struct mounts *next;    	/* forward link */
 };
 
+#define	X_BADFILEOPS	"badfileops"
+extern KA_T X_bfopsa;
 #define	X_NCACHE	"ncache"
 #define	X_NCSIZE	"ncsize"
 #define	NL_NAME		n_name
 
 extern int Np;				/* number of kernel processes */
 
-# if	FREEBSDV>=2000
 extern struct kinfo_proc *P;		/* local process table copy */
-# endif	/* FREEBSDV>=2000 */
 
 struct sfile {
 	char *aname;			/* argument file name */
@@ -586,15 +508,8 @@ struct sfile {
 
 };
 
-# if	FREEBSDV==4100 || FREEBSDV==4110
-#define	XDR_VOID	(xdrproc_t)xdr_void
-#define	XDR_PMAPLIST	(xdrproc_t)xdr_pmaplist
-# endif	/* FREEBSDV==4100 || FREEBSDV==4110 */
-
-# if	FREEBSDV>=5000
 #define	XDR_VOID	(const xdrproc_t)xdr_void
 #define	XDR_PMAPLIST	(const xdrproc_t)xdr_pmaplist
-# endif	/* FREEBSDV>=5000 */
 
 
 /*
@@ -611,9 +526,9 @@ struct sfile {
 
 # if     defined(HASNCACHE)
 #include <sys/uio.h>
-#  if	FREEBSDV<4000 || (FREEBSDV>=4000 && defined(HASNAMECACHE))
+#  if	defined(HASNAMECACHE)
 #include <sys/namei.h>
-#  else	/* FREEBSDV>=4000 && !defined(HASNAMECACHE) */
+#  else	/* !defined(HASNAMECACHE) */
 /*
  * The namecache struct definition should come from a header file that
  * can be #include'd, but it has been moved to a kernel source file in
@@ -623,9 +538,15 @@ struct sfile {
  */
 
 struct	namecache {
+#   if  __FreeBSD_version < 1202000 || (__FreeBSD_version >= 1300000 && __FreeBSD_version < 1300105)
 	LIST_ENTRY(namecache) nc_hash;	/* hash chain */
 	LIST_ENTRY(namecache) nc_src;	/* source vnode list */
 	TAILQ_ENTRY(namecache) nc_dst;	/* destination vnode list */
+#   else
+	LIST_ENTRY(namecache) nc_src;   /* source vnode list */
+	TAILQ_ENTRY(namecache) nc_dst;  /* destination vnode list */
+	LIST_ENTRY(namecache) nc_hash;  /* hash chain */
+#   endif
 	struct	vnode *nc_dvp;		/* vnode of parent of name */
 	struct	vnode *nc_vp;		/* vnode the name refers to */
 	u_char	nc_flag;		/* flag bits */
@@ -642,22 +563,14 @@ struct	namecache {
 					 * suggesting the guess is a safe one.
 					 * (VAA, 10 Apr 2002) */
 };
-#  endif	/* FREEBSDV<4000 || (FREEBSDV>=4000 && defined(HASNAMECACHE)) */
+#  endif	/* defined(HASNAMECACHE)) */
 
 #define	NCACHE		namecache	/* kernel's structure name */
 #define	NCACHE_NM	nc_name		/* name in NCACHE */
 #define	NCACHE_NMLEN	nc_nlen		/* name length in NCACHE */
 
-#  if	FREEBSDV<2005
-#define	NCACHE_NXT	nc_nxt		/* link in NCACHE */
-#  else	/* FREEBSDV>=2005 */
-#   if	FREEBSDV<2010
-#define	NCACHE_NXT	nc_lru.tqe_next	/* link in NCACHE */
-#   else	/* FREEBSDV>=2010 */
 #include <stddef.h>
 #define	NCACHE_NXT	nc_hash.le_next	/* link in NCACHE */
-#   endif	/* FREEBSDV<2010 */
-#  endif	/* FREEBSDV<2005 */
 
 #define	NCACHE_NODEADDR	nc_vp		/* node address in NCACHE */
 #define	NCACHE_PARADDR	nc_dvp		/* parent node address in NCACHE */
@@ -668,9 +581,7 @@ struct	namecache {
 #  endif	/* DEFINED(HASNCVPID) */
 # endif  /* defined(HASNCACHE) */
 
-# if	FREEBSDV>=5000
 #define	VNODE_VFLAG	v_iflag
 #define	NCACHE_VROOT	VV_ROOT
-# endif	/* FREEBSDV>=5000 */
 
 #endif	/* defined(FREEBSD_LSOF_H) */
