@@ -32,7 +32,6 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dmnt.c,v 1.11 2005/05/11 12:52:50 abe Exp $";
 #endif
 
 
@@ -48,53 +47,91 @@ static int Lmist = 0;					/* Lmi status */
 
 #undef	HAS_MNT_NAMES
 
-#if	FREEBSDV<2000
-static char *mnt_names[] = { "none", "ufs", "nfs", "mfs", "pc", "iso9600",
-			     "procfs", "devfs" };
-#define	HAS_MNT_NAMES	1
-#else	/* FREEBSDV>=2000 */
 # if	defined(MOUNT_NONE)
 static char *mnt_names[] = INITMOUNTNAMES;
 #define	HAS_MNT_NAMES	1
 # endif	/* defined(MOUNT_NONE)) */
-#endif	/* FREEBSDV<2000 */
 
 
-#if	FREEBSDV>=5000 && defined(HAS_NO_SI_UDEV)
+#if	defined(HAS_NO_SI_UDEV)
 /*
- * dev2udev() -- convert a kernel device number to a user device number
+ * Dev2Udev() -- convert a kernel device number to a user device number
  */
 
 dev_t
-dev2udev(c)
+Dev2Udev(c)
+
+# if	defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV)
+	KA_T c;
+# else	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
 	struct cdev *c;
+# endif	/* defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV) */
+
 {
+
+# if	!defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV)
+	char *cp;
 	char *dn = (char *)NULL;
 	char *ln = (char *)NULL;
 	struct statfs *mb;
-	struct mounts *mtp;
 	int n, sr;
 	static u_int s;
 	struct stat sb;
-	static ss = 0;
+	static int ss = 0;
+# endif	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
 
-	if (ss)
+# if	defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV)
+	KA_T ca;
+	struct cdev_priv cp;
+
+	if (!c)
+	    return(NODEV);
+
+#  if	defined(HAS_CDEV2PRIV)
+	ca = (KA_T)cdev2priv((struct cdev *)c);
+#  else	/* !defined(HAS_CDEV2PRIV) */
+	ca = (KA_T)member2struct(cdev_priv, cdp_c, c);
+#  endif	/* defined(HAS_CDEV2PRIV) */
+
+	if (kread((KA_T)ca, (char *)&cp, sizeof(cp)))
+	    return(NODEV);
+	return((dev_t)cp.cdp_inode);
+# else	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
+#  if	defined(HAS_SI_PRIV)
+/*
+ * If the cdev structure has a private sub-structure, read it.
+ */
+	struct cdev_priv sp;
+
+	if (!c->si_priv || kread((KA_T)c->si_priv, (char *)&sp, sizeof(sp)))
+	    return(0);
+#  endif	/* defined(HAS_SI_PRIV) */
+
+	if (ss) {
+
+#  if	defined(HAS_SI_PRIV)
+	    return(sp.cdp_inode ^ s);
+#  else	/* !defined(HAS_SI_PRIV) */
 	    return(c->si_inode ^ s);
+#  endif	/* defined(HAS_SI_PRIV) */
+
+	}
+
 /*
  * Determine the random udev seed from stat(2) operations on "/" and
  * its device.
  */
 	if ((n = getmntinfo(&mb, MNT_NOWAIT)) <= 0) {
 	    (void) fprintf(stderr, "%s: no mount information\n", Pn);
-	    Exit(1);
+	    Error();
 	}
 	for (; n; n--, mb++) {
 
-#if	defined(MOUNT_NONE)
+# if	defined(MOUNT_NONE)
 	    if (mb->f_type == MOUNT_NONE || mb->f_type >= MOUNT_MAXTYPE)
-#else	/* !defined(MOUNT_NONE) */
+# else	/* !defined(MOUNT_NONE) */
 	    if (!mb->f_type)
-#endif	/* defined(MOUNT_NONE) */
+# endif	/* defined(MOUNT_NONE) */
 
 		continue;
 	/*
@@ -105,14 +142,14 @@ dev2udev(c)
 		(void) free((FREE_P *)dn);
 	    if (!(dn = mkstrcpy(mb->f_mntonname, (MALLOC_S *)NULL))) {
 
-dev2udev_no_space:
+Dev2Udev_no_space:
 
 		(void) fprintf(stderr, "%s: no space for mount at ", Pn);
 		safestrprt(mb->f_mntonname, stderr, 0);
 		(void) fprintf(stderr, " (");
 		safestrprt(mb->f_mntfromname, stderr, 0);
 		(void) fprintf(stderr, ")\n");
-		Exit(1);
+		Error();
 	    }
 	    if (!(ln = Readlink(dn))) {
 		if (!Fwarn) {
@@ -135,18 +172,34 @@ dev2udev_no_space:
 	 */
 	    (void) free((FREE_P *)dn);
 	    if (!(dn = mkstrcpy(mb->f_mntfromname, (MALLOC_S *)NULL)))
-		goto dev2udev_no_space;
+		goto Dev2Udev_no_space;
 	    ln = Readlink(dn);
-	    sr = statsafely(ln, &sb);
+	    if ((sr = statsafely(ln, &sb))) {
+
+	    /*
+	     * If the device stat(2) failed, see if the device name indicates
+	     * an NFS mount, a cd9660 device, or a ZFS mount.  If any condition
+	     * is true, set the user device number seed to zero.
+	     */
+		if (((cp = strrchr(ln, ':')) && (*(cp + 1) == '/'))
+		||  !strcasecmp(mb->f_fstypename, "cd9660")
+		||  !strcasecmp(mb->f_fstypename, "zfs")
+		) {
+		    ss = 1;
+		    s = (u_int)0;
+		}
+	    }
 	    if (ln != dn)
 		(void) free((FREE_P *)ln);
 	    ln = (char *)NULL;
 	    (void) free((FREE_P *)dn);
 	    dn = (char *)NULL;
-	    if (sr)
+	    if (sr && !ss)
 		continue;
-	    ss = 1;
-	    s = (u_int)sb.st_ino ^ (u_int)sb.st_rdev;
+	    if (!ss) {
+		ss = 1;
+		s = (u_int)sb.st_ino ^ (u_int)sb.st_rdev;
+	    }
 	    break;
 	}
 /*
@@ -160,12 +213,22 @@ dev2udev_no_space:
  * If the device seed is known, return its application to the cdev structure's
  * inode.
  */
-	if (ss)
+	if (ss) {
+
+#  if	defined(HAS_SI_PRIV)
+	    return(sp.cdp_inode ^ s);
+#  else	/* !defined(HAS_SI_PRIV) */
 	    return(c->si_inode ^ s);
+#  endif	/* defined(HAS_SI_PRIV) */
+
+	}
 	(void) fprintf(stderr, "%s: can't determine user device random seed.\n",	    Pn);
-	Exit(1);
+	Error();
+
+# endif	/* !defined(HAS_CONF_MINOR) */
+
 }
-#endif	/* FREEBSDV>=5000 && defined(HAS_NO_SI_UDEV) */
+#endif	/* defined(HAS_NO_SI_UDEV) */
 
 
 /*
@@ -221,7 +284,7 @@ no_space_for_mount:
 		(void) fprintf(stderr, " (");
 		safestrprt(mb->f_mntfromname, stderr, 0);
 		(void) fprintf(stderr, ")\n");
-		Exit(1);
+		Error();
 	    }
 	    if (!(ln = Readlink(dn))) {
 		if (!Fwarn) {
@@ -259,8 +322,8 @@ no_space_for_mount:
 		sb.st_mode = S_IFDIR | 0777;
 		if (!Fwarn) {
 		    (void) fprintf(stderr,
-			"      assuming \"dev=%x\" from mount table\n",
-			sb.st_dev);
+			"      assuming \"dev=%lx\" from mount table\n",
+			(unsigned long)sb.st_dev);
 		}
 	    }
 	/*
@@ -296,7 +359,7 @@ no_space_for_mount:
 	    mtp->next = Lmi;
 	    mtp->dev = sb.st_dev;
 	    mtp->rdev = sb.st_rdev;
-	    mtp->inode = sb.st_ino;
+	    mtp->inode = (INODETYPE)sb.st_ino;
 	    mtp->mode = sb.st_mode;
 	/*
 	 * Interpolate a possible file system (mounted-on) device name link.
@@ -351,14 +414,14 @@ readvfs(vm)
 	if (!(vp = (struct l_vfs *)malloc(sizeof(struct l_vfs)))) {
 	    (void) fprintf(stderr, "%s: PID %d, no space for vfs\n",
 		Pn, Lp->pid);
-	    Exit(1);
+	    Error();
 	}
 	if (!(vp->dir = mkstrcpy(m.m_stat.f_mntonname, (MALLOC_S *)NULL))
 	||  !(vp->fsname = mkstrcpy(m.m_stat.f_mntfromname, (MALLOC_S *)NULL)))
 	{
 	    (void) fprintf(stderr, "%s: PID %d, no space for mount names\n",
 		Pn, Lp->pid);
-	    Exit(1);
+	    Error();
 	}
 	vp->addr = vm;
 	vp->fsid = m.m_stat.f_fsid;
@@ -379,7 +442,7 @@ readvfs(vm)
 		    (void) fprintf(stderr,
 			"%s: no space for fs type name: ", Pn);
 		    safestrprt(m.m_stat.f_fstypename, stderr, 1);
-		    Exit(1);
+		    Error();
 		}
 	    } else
 		vp->typnm = "";
