@@ -32,7 +32,6 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.41 2011/08/07 22:51:28 abe Exp $";
 #endif
 
 
@@ -51,11 +50,21 @@ static char *rcsid = "$Id: dnode.c,v 1.41 2011/08/07 22:51:28 abe Exp $";
 _PROTOTYPE(static int lkup_dev_tty,(dev_t *dr, INODETYPE *ir));
 #endif	/* defined(HASFDESCFS) && HASFDESCFS==1 */
 
+
+#if	defined(HASPTSFN) && defined(DTYPE_PTS)
+#include <sys/tty.h>
+#endif	/* defined(HASPTSFN) && defined(DTYPE_PTS) */
+
+
+#if	defined(HAS_TMPFS)
+#include <fs/tmpfs/tmpfs.h>
+#endif	/* defined(HAS_TMPFS) */
+
 _PROTOTYPE(static void get_lock_state,(KA_T f));
 
 
 /*
- * get_lock_state() -- get the lock state
+ * get_lock_state() - get the lock state
  */
 
 static void
@@ -145,7 +154,6 @@ get_lock_state(f)
 }
 
 
-#if	FREEBSDV>=2000
 # if	defined(HASPROCFS)
 _PROTOTYPE(static void getmemsz,(pid_t pid));
 
@@ -174,7 +182,6 @@ getmemsz(pid)
 	}
 }
 # endif	/* defined(HASPROCFS) */
-#endif	/* FREEBSDV>=2000 */
 
 
 #if	defined(HASFDESCFS) && HASFDESCFS==1
@@ -225,7 +232,7 @@ lkup_dev_tty_again:
 
 #if	defined(HASKQUEUE)
 /*
- * process_kqueue() -- process kqueue file
+ * process_kqueue() - process kqueue file
  *
  * Strictly speaking this function should appear in dfile.c, because it is
  * a file processing function.  However, the Net and Open BSD sources don't
@@ -265,30 +272,17 @@ process_node(va)
 	struct nfsnode *n;
 	size_t sz;
 	char *ty;
+	unsigned char ums;
 	enum vtype type;
 	struct vnode *v, vb;
 	struct l_vfs *vfs;
 
-#if	FREEBSDV>=2000
 	struct inode ib;
 	struct nfsnode nb;
-# if	FREEBSDV>=4000
-#  if	FREEBSDV<5000
-	struct specinfo si;
-#  else	/* FREEBSDV>=5000 */
 #   if	!defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV)
 	struct cdev si;
 #   endif	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
-#  endif	/* FREEBSDV<5000 */
-# endif	/* FREEBSDV>=4000 */
-#endif	/* FREEBSDV>=2000 */
 
-#if	FREEBSDV<5000
-	struct mfsnode *m;
-# if	FREEBSDV>=2000
-	struct mfsnode mb;
-# endif	/* FREEBSDV>=2000 */
-#endif	/* FREEBSDV<5000 */
 
 #if	defined(HAS9660FS)
 	dev_t iso_dev;
@@ -307,13 +301,26 @@ process_node(va)
 	static int f_tty_s = 0;
 # endif	/* HASFDESCFS==1 */
 
-# if	FREEBSDV>=2000
 	struct fdescnode fb;
-# endif	/* FREEBSDV>=2000 */
 
 #endif	/* defined(HASFDESCFS) */
 
-#if	FREEBSDV>=5000
+#if	defined(HASFUSEFS)
+	dev_t fuse_dev;
+	int fuse_dev_def, fuse_stat;
+	INODETYPE fuse_ino;
+	long fuse_links;
+	SZOFFTYPE fuse_sz;
+#endif	/* defined(HASFUSEFS) */
+
+#if	defined(HASMSDOSFS)
+	dev_t msdos_dev;
+	int msdos_dev_def, msdos_stat;
+	INODETYPE msdos_ino;
+	long msdos_links;
+	SZOFFTYPE msdos_sz;
+#endif	/* defined(HASMSDOSFS) */
+
 # if	defined(HAS_UFS1_2)
 	int ufst;
 	struct ufsmount um;
@@ -330,10 +337,15 @@ process_node(va)
 	struct devfs_dirent *d;
 	char vtbuf[32];
 	char *vtbp;
-	enum vtagtype { VT_DEVFS, VT_FDESC, VT_ISOFS, VT_PSEUDOFS, VT_NFS,
-			VT_NULL, VT_UFS, VT_ZFS, VT_UNKNOWN
+	enum vtagtype { VT_DEVFS, VT_FDESC, VT_FUSEFS, VT_ISOFS, VT_PSEUDOFS,
+			VT_NFS, VT_NULL, VT_TMPFS, VT_UFS, VT_ZFS, VT_MSDOSFS,
+			VT_UNKNOWN
 		      };
-#endif	/* FREEBSDV>=5000 */
+
+# if	defined(HAS_TMPFS)
+	struct tmpfs_node tn;
+	struct tmpfs_node *tnp;
+# endif	/* defined(HAS_TMPFS) */
 
 #if	defined(HASNULLFS)
 # if	!defined(HASPRINTDEV)
@@ -350,12 +362,10 @@ process_node(va)
 	static int pgsz = -1;
 	struct vmspace vm;
 
-# if	FREEBSDV>=2000
 	struct pfsnode pb;
-# endif	/* FREEBSDV>=2000 */
 #endif	/* defined(HASPROCFS) */
 
-#if	defined(HASPSEUDOFS) 
+#if	defined(HASPSEUDOFS)
 	struct pfs_node pn;
 	struct pfs_node *pnp;
 #endif	/* defined(HASPSEUDOFS) */
@@ -388,7 +398,7 @@ process_overlaid_node:
  * Initialize miscellaneous variables.  This is done so that processing an
  * overlaid node will be a fresh start.
  */
-	devs = rdevs = 0;
+	devs = rdevs = ums = 0;
 	i = (struct inode *)NULL;
 	n = (struct nfsnode *)NULL;
 	Namech[0] = '\0';
@@ -401,24 +411,33 @@ process_overlaid_node:
 	f = (struct fdescnode *)NULL;
 #endif	/* defined(HASFDESCFS) */
 
-#if	FREEBSDV<5000
-	m = (struct mfsnode *)NULL;
-#else	/* FREEBSDV>=5000 */
+#if	defined(HASFUSEFS)
+	fuse_dev_def = fuse_stat = 0;
+#endif	/* defined(HASFUSEFS) */
+
+#if	defined(HASMSDOSFS)
+	msdos_dev_def = msdos_stat = 0;
+#endif /* defined(HASMSDOSFS) */
+
 	cds = 0;
 	d = (struct devfs_dirent *)NULL;
 # if	defined(HAS_UFS1_2)
 	ufst = 0;
 # endif	/* !defined(HAS_UFS1_2) */
-#endif	/* FREEBSDV<5000 */
 
 #if	defined(HASPROCFS)
 	p = (struct pfsnode *)NULL;
 #endif	/* defined(HASPROCFS) */
 
-#if	defined(HASPSEUDOFS) 
+#if	defined(HASPSEUDOFS)
 	pnp = (struct pfs_node *)NULL;
 #endif	/* defined(HASPSEUDOFS) */
 
+# if	defined(HAS_TMPFS)
+	tnp = (struct tmpfs_node *)NULL;
+# endif	/* defined(HAS_TMPFS) */
+
+
 #if	defined(HAS_ZFS)
 	z = (zfs_info_t *)NULL;
 	zm = (char *)NULL;
@@ -483,6 +502,11 @@ process_overlaid_node:
 		else if (strcasecmp(vfs->typnm, "pseudofs") == 0)
 		    Ntype = N_PSEU;
 # endif	/* defined(HASPSEUDOFS) */
+
+# if	defined(HAS_TMPFS)
+		else if (strcasecmp(vfs->typnm, "tmpfs") == 0)
+		    Ntype = N_TMP;
+# endif	/* defined(HAS_TMPFS) */
 #endif	/* defined(MOUNT_NONE) */
 
 	    }
@@ -497,7 +521,6 @@ process_overlaid_node:
 	    }
 	}
 
-#if	FREEBSDV>=5000
 /*
  * For FreeBSD 5 and above VCHR and VBLK vnodes get the v_rdev structure.
  */
@@ -511,18 +534,16 @@ process_overlaid_node:
 	) {
 	    cds = 1;
 	}
-#endif	/* FREEBSDV>=5000 */
 
 /*
  * Define the specific node pointer.
  */
 
-#if	FREEBSDV>=5000
 /*
  * Get the pseudo vnode tag type for FreeBSD >= 5.
  */
 	vtag = VT_UNKNOWN;
-	if (v->v_tag && !kread((KA_T)v->v_tag, (char *)&vtbuf, sizeof(vtbuf)))
+	if (!kread((KA_T)v->v_lock.lock_object.lo_name, (char *)&vtbuf, sizeof(vtbuf)))
 	{
 	    vtbuf[sizeof(vtbuf) - 1] = '\0';
 	    vtbp = vtbuf;
@@ -547,23 +568,31 @@ process_overlaid_node:
 		vtag = VT_DEVFS;
 	    else if (!strcmp(vtbuf, "nfs"))
 		vtag = VT_NFS;
+	    else if (!strcmp(vtbuf, "newnfs"))
+		vtag = VT_NFS;
+	    else if (!strcmp(vtbuf, "oldnfs"))
+		vtag = VT_NFS;
 	    else if (!strcmp(vtbuf, "isofs"))
 		vtag = VT_ISOFS;
 	    else if (!strcmp(vtbuf, "pseudofs"))
 		vtag = VT_PSEUDOFS;
+	    else if (!strcmp(vtbuf, "nullfs"))
+		vtag = VT_NULL;
 	    else if (!strcmp(vtbuf, "null"))
 		vtag = VT_NULL;
 	    else if (!strcmp(vtbuf, "fdesc"))
 		vtag = VT_FDESC;
+	    else if (!strcmp(vtbuf, "fuse"))
+		vtag = VT_FUSEFS;
+	    else if (!strcmp(vtbuf, "tmpfs"))
+		vtag = VT_TMPFS;
+	    else if (!strcmp(vtbuf, "msdosfs"))
+		vtag = VT_MSDOSFS;
 	} else
 	    vtbp = "(unknown)";
-#else	/* FREEBSDV<5000 */
-	vtag = v->v_tag;
-#endif	/* FREEBSDV>=5000 */
 
 	switch (vtag) {
 
-#if	FREEBSDV>=5000
 	case VT_DEVFS:
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&de, sizeof(de)))
@@ -584,14 +613,10 @@ process_overlaid_node:
 		}
 	    }
 	    break;
-#endif	/* FREEBSDV>=5000 */
 
 #if	defined(HASFDESCFS)
 	case VT_FDESC:
 
-# if	FREEBSDV<2000
-	    f = (struct fdescnode *)v->v_data;
-# else	/* FREEBSDV>=2000 */
 	    if (kread((KA_T)v->v_data, (char *)&fb, sizeof(fb)) != 0) {
 		(void) snpf(Namech, Namechl, "can't read fdescnode at: %s",
 		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
@@ -600,9 +625,36 @@ process_overlaid_node:
 	    }
 	    f = &fb;
 	    break;
-# endif	/* FREEBSDV<2000 */
 #endif	/* defined(HASFDESCFS) */
 
+#if	defined(HASFUSEFS)
+	case VT_FUSEFS:
+	    if (read_fuse_node(v, &fuse_dev, &fuse_dev_def, &fuse_ino,
+			       &fuse_links, &fuse_sz))
+	    {
+		(void) snpf(Namech, Namechl, "no fuse node: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
+		enter_nm(Namech);
+		return;
+	    }
+	    fuse_stat = 1;
+	    break;
+#endif	/* defined(HASFUSEFS) */
+
+#if	defined(HASMSDOSFS)
+	case VT_MSDOSFS:
+	    if (read_msdos_node(v, &msdos_dev, &msdos_dev_def, &msdos_ino,
+				&msdos_links, &msdos_sz))
+	    {
+		(void) snpf(Namech, Namechl, "can't read denode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
+		enter_nm(Namech);
+		return;
+	    }
+	    msdos_stat = 1;
+	    break;
+#endif	/* defined(HASMSDOSFS) */
+
 #if	defined(HAS9660FS)
 	case VT_ISOFS:
 	    if (read_iso_node(v, &iso_dev, &iso_dev_def, &iso_ino, &iso_links,
@@ -617,29 +669,9 @@ process_overlaid_node:
 	    break;
 #endif	/* defined(HAS9660FS) */
 
-#if	FREEBSDV<5000
-	case VT_MFS:
-
-# if	FREEBSDV<2000
-	    m = (struct mfsnode *)v->v_data;
-# else	/* FREEBSDV>=2000 */
-	    if (!v->v_data
-	    ||  kread((KA_T)v->v_data, (char *)&mb, sizeof(mb))) {
-		(void) snpf(Namech, Namechl, "no mfs node: %s",
-		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
-		enter_nm(Namech);
-		return;
-	    }
-	    m = &mb;
-# endif	/* FREEBSDV<2000 */
-#endif	/* FREEBSDV<5000 */
-
 	    break;
 	case VT_NFS:
 
-#if	FREEBSDV<2000
-	    n = (struct nfsnode *)v->v_data;
-#else	/* FREEBSDV>=2000 */
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&nb, sizeof(nb))) {
 		(void) snpf(Namech, Namechl, "no nfs node: %s",
@@ -648,7 +680,6 @@ process_overlaid_node:
 		return;
 	    }
 	    n = &nb;
-#endif	/* FREEBSDV<2000 */
 
 	    break;
 
@@ -709,9 +740,6 @@ process_overlaid_node:
 #if	defined(HASPROCFS)
 	case VT_PROCFS:
 
-# if	FREEBSDV<2000
-	    p = (struct pfsnode *)v->v_data;
-# else	/* FREEBSDV>=2000 */
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&pb, sizeof(pb))) {
 		(void) snpf(Namech, Namechl, "no pfs node: %s",
@@ -720,7 +748,6 @@ process_overlaid_node:
 		return;
 	    }
 	    p = &pb;
-# endif	/* FREEBSDV<2000 */
 
 	    break;
 #endif	/* defined(HASPROCFS) */
@@ -738,11 +765,21 @@ process_overlaid_node:
 	    break;
 #endif	/* defined(HASPSEUDOFS) */
 
+# if	defined(HAS_TMPFS)
+	case VT_TMPFS:
+	    if (!v->v_data
+	    ||  kread((KA_T)v->v_data, (char *)&tn, sizeof(tn))) {
+		(void) snpf(Namech, Namechl, "no tmpfs_node: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
+		enter_nm(Namech);
+		return;
+	    }
+	    tnp = &tn;
+	    break;
+# endif	/* defined(HAS_TMPFS) */
+
 	case VT_UFS:
 
-#if	FREEBSDV<2000
-	    i = (struct inode *)v->v_data;
-#else	/* FREEBSDV>=2000 */
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&ib, sizeof(ib))) {
 		(void) snpf(Namech, Namechl, "no ufs node: %s",
@@ -754,6 +791,7 @@ process_overlaid_node:
 
 # if	defined(HAS_UFS1_2)
 	    if (i->i_ump && !kread((KA_T)i->i_ump, (char *)&um, sizeof(um))) {
+		ums =  1;
 		if (um.um_fstype == UFS1) {
 		    if (i->i_din1
 		    &&  !kread((KA_T)i->i_din1, (char *)&d1, sizeof(d1)))
@@ -765,7 +803,6 @@ process_overlaid_node:
 		}
 	    }
 # endif	/* defined(HAS_UFS1_2) */
-#endif	/* FREEBSDV<2000 */
 
 #if	defined(HAS_V_LOCKF)
 	    if (v->v_lockf)
@@ -779,6 +816,7 @@ process_overlaid_node:
 
 #if	defined(HAS_ZFS)
 	case VT_ZFS:
+	    memset((void *)&zi, 0, sizeof(zfs_info_t));
 	    if (!v->v_data
 	    ||  (zm = readzfsnode((KA_T)v->v_data, &zi,
 				  ((v->v_vflag & VV_ROOT) ? 1 : 0)))
@@ -805,11 +843,7 @@ process_overlaid_node:
 	    if (v->v_type == VBAD || v->v_type == VNON)
 		break;
 
-#if	FREEBSDV<5000
-	    (void) snpf(Namech,Namechl,"unknown file system type: %d",v->v_tag);
-#else	/* FREEBSDV>=5000 */
 	    (void) snpf(Namech, Namechl, "unknown file system type: %s", vtbp);
-#endif	/* FREEBSDV<5000 */
 
 	    enter_nm(Namech);
 	    return;
@@ -827,35 +861,36 @@ process_overlaid_node:
 	    }
 	} else if (i) {
 
-#if	FREEBSDV>=4000
+# if	defined(HAS_NO_IDEV)
+	    if (ums) {
+		dev = Dev2Udev((KA_T)um.um_dev);
+		devs = 1;
+	    }
+# else	/* !defined(HAS_NO_IDEV) */
 	    if (i->i_dev
 
-# if	!defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV)
+#  if	!defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV)
 	    &&  !kread((KA_T)i->i_dev, (char *)&si, sizeof(si))
-# endif	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
+#  endif/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
 
 	    ) {
 
-# if	defined(HAS_NO_SI_UDEV)
-#  if	defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV)
+#  if	defined(HAS_NO_SI_UDEV)
+#   if	defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV)
 		dev = Dev2Udev((KA_T)i->i_dev);
-#  else	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
+#   else	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
 		dev = Dev2Udev(&si);
-#  endif	/* defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV) */
-# else	/* !defined(HAS_NO_SI_UDEV) */
+#   endif	/* defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV) */
+#   else	/* !defined(HAS_NO_SI_UDEV) */
 		dev = si.si_udev;
-# endif	/* defined(HAS_NO_SI_UDEV) */
+#  endif	/* defined(HAS_NO_SI_UDEV) */
 
 		devs = 1;
 	    }
-#else	/* FREEBSDV<4000 */
-	    dev = i->i_dev;
-	    devs = 1;
-#endif	/* FREEBSDV>=4000 */
+# endif	/* defined(HAS_NO_IDEV) */
 
 	    if ((type == VCHR) || (type == VBLK)) {
 
-#if	FREEBSDV>=5000
 # if	defined(HAS_UFS1_2)
 		if (ufst == 1) {
 		    rdev = d1.di_rdev;
@@ -880,11 +915,6 @@ process_overlaid_node:
 
 		    rdevs = 1;
 		}
-#else	/* FREEBSDV<5000 */
-		rdev = i->i_rdev;
-		rdevs = 1;
-#endif	/* FREEBSDV>=5000 */
-
 	    }
 	}
 
@@ -895,7 +925,14 @@ process_overlaid_node:
 	 * Record information returned by readzfsnode().
 	 */
 	    if (vfs) {
-		dev = vfs->fsid.val[0];
+		union {
+		    int32_t val[2];
+		    dev_t dev;
+		} vfs_fsid;
+
+		vfs_fsid.val[0] = vfs->fsid.val[0];
+		vfs_fsid.val[1] = vfs->fsid.val[1];
+		dev = vfs_fsid.dev;
 		devs = 1;
 	    }
 	    if ((type == VCHR) || (type == VBLK)) {
@@ -917,7 +954,7 @@ process_overlaid_node:
 
 #  if	HASFDESCFS==1
 	    else
-#  endif	/* HASFDESFS==1 */
+#  endif	/* HASFDESCFS==1 */
 # endif	/* defined(HASFDLINK) */
 
 # if	HASFDESCFS==1
@@ -930,7 +967,7 @@ process_overlaid_node:
 			devs = Lf->inp_ty = 1;
 		    }
 		}
-# endif	/* HASFDESFS==1 */
+# endif	/* HASFDESCFS==1 */
 
 	}
 #endif	/* defined(HASFDESCFS) && (defined(HASFDLINK) || HASFDESCFS==1) */
@@ -942,7 +979,20 @@ process_overlaid_node:
 	}
 #endif	/* defined(HAS9660FS) */
 
-#if	FREEBSDV>=5000
+#if	defined(HASFUSEFS)
+	else if (fuse_stat && fuse_dev_def) {
+	    dev = fuse_dev;
+	    devs = Lf->inp_ty = 1;
+	}
+#endif	/* defined(HASFUSEFS) */
+
+#if	defined(HASMSDOSFS)
+	else if (msdos_stat && msdos_dev_def) {
+	    dev = msdos_dev;
+	    devs = Lf->inp_ty = 1;
+	}
+#endif	/* defined(HASMSDOSFS) */
+
 	else if (d) {
 	    if (vfs) {
 		dev = vfs->fsid.val[0];
@@ -951,7 +1001,7 @@ process_overlaid_node:
 		dev = DevDev;
 		devs = 1;
 	    }
-	    if ((type == VCHR)) {
+	    if (type == VCHR) {
 
 # if	defined(HAS_UFS1_2)
 		if (ufst == 1) {
@@ -979,7 +1029,6 @@ process_overlaid_node:
 		}
 	    }
 	}
-#endif	/* FREEBSDV>=5000 */
 
 #if	defined(HASPSEUDOFS)
 	else if (pnp) {
@@ -990,6 +1039,19 @@ process_overlaid_node:
 	}
 #endif	/* defined(HASPSEUDOFS) */
 
+# if	defined(HAS_TMPFS)
+	else if (tnp) {
+	    if (vfs) {
+		dev = vfs->fsid.val[0];
+		devs = 1;
+	    }
+	    if (tnp->tn_type == VBLK || tnp->tn_type == VCHR) {
+		rdev = tnp->tn_rdev;
+		rdevs = 1;
+	    }
+	}
+# endif	/* defined(HAS_TMPFS) */
+
 /*
  * Obtain the inode number.
  */
@@ -1019,13 +1081,25 @@ process_overlaid_node:
 	}
 #endif	/* defined(HAS9660FS) */
 
+#if	defined(HASFUSEFS)
+	else if (fuse_stat) {
+	    Lf->inode = fuse_ino;
+	    Lf->inp_ty = 1;
+	}
+#endif	/* defined(HASFUSEFS) */
+
+#if	defined(HASMSDOSFS)
+	else if (msdos_stat) {
+	    Lf->inode = msdos_ino;
+	    Lf->inp_ty = 1;
+	}
+#endif	/* defined(HASMSDOSFS) */
+
 #if	defined(HASPROCFS)
-# if	FREEBSDV>=2000
 	else if (p) {
 	    Lf->inode = (INODETYPE)p->pfs_fileno;
 	    Lf->inp_ty = 1;
 	}
-# endif	/* FREEBSDV>=2000 */
 #endif	/* defined(HASPROCFS) */
 
 #if	defined(HASPSEUDOFS)
@@ -1035,12 +1109,17 @@ process_overlaid_node:
 	}
 #endif	/* defined(HASPSEUDOFS) */
 
-#if	FREEBSDV>=5000
 	else if (d) {
 	    Lf->inode = (INODETYPE)d->de_inode;
 	    Lf->inp_ty = 1;
 	}
-#endif	/* FREEBSDV>=5000 */
+
+# if	defined(HAS_TMPFS)
+	else if (tnp) {
+	    Lf->inode = (INODETYPE)tnp->tn_id;
+	    Lf->inp_ty = 1;
+	}
+# endif	/* defined(HAS_TMPFS) */
 
 /*
  * Obtain the file size.
@@ -1063,18 +1142,6 @@ process_overlaid_node:
 #if	defined(HASPROCFS)
 	    case N_PROC:
 
-# if	FREEBSDV<2000
-		if (type == VDIR || !p || !p->pfs_vs
-		||  kread((KA_T)p->pfs_vs, (char *)&vm, sizeof(vm)))
-		    break;
-		if (pgsz < 0)
-		    pgsz = getpagesize();
-		Lf->sz = (SZOFFTYPE)((pgsz * vm.vm_tsize)
-		       +         (pgsz * vm.vm_dsize)
-		       +         (pgsz * vm.vm_ssize));
-		Lf->sz_def = 1;
-		break;
-# else	/* FREEBSDV>=2000 */
 		if (p) {
 		    switch(p->pfs_type) {
 		    case Proot:
@@ -1095,7 +1162,6 @@ process_overlaid_node:
 			break;
 		    }
 		}
-# endif	/* FREEBSDV<2000 */
 #endif	/* defined(HASPROCFS) */
 
 #if	defined(HASPSEUDOFS)
@@ -1131,24 +1197,44 @@ process_overlaid_node:
 		    }
 #endif  /* defined(HAS_ZFS) */
 
-#if	FREEBSDV<5000
-		    else if (m) {
-			Lf->sz = (SZOFFTYPE)m->mfs_size;
-			Lf->sz_def = 1;
-		    }
-#endif	/* FREEBSDV<5000 */
-
 #if	defined(HAS9660FS)
 		    else if (iso_stat) {
 			Lf->sz = (SZOFFTYPE)iso_sz;
 			Lf->sz_def = 1;
 		    }
+
 #endif	/* defined(HAS9660FS) */
 
+#if	defined(HASFUSEFS)
+		    else if (fuse_stat) {
+			Lf->sz = (SZOFFTYPE)fuse_sz;
+			Lf->sz_def = 1;
+		    }
+#endif	/* defined(HASFUSEFS) */
+
+#if	defined(HASMSDOSFS)
+		    else if (msdos_stat) {
+			Lf->sz = (SZOFFTYPE)msdos_sz;
+			Lf->sz_def = 1;
+		    }
+#endif	/* defined(HASMSDOSFS) */
 		}
 		else if ((type == VCHR || type == VBLK) && !Fsize)
 		    Lf->off_def = 1;
+		break;
+
+# if	defined(HAS_TMPFS)
+		case N_TMP:
+		    if ((tnp->tn_type == VBLK || tnp->tn_type == VCHR)
+		    &&  !Fsize) {
+			Lf->off_def = 1;
+		    } else {
+			Lf->sz = (SZOFFTYPE)tnp->tn_size;
+			Lf->sz_def = 1;
+		    }
 		    break;
+# endif	/* defined(HAS_TMPFS) */
+
 	    }
 	}
 /*
@@ -1190,12 +1276,24 @@ process_overlaid_node:
 		}
 #endif	/* defined(HAS9660FS) */
 
-#if	FREEBSDV>=5000
+#if	defined(HASFUSEFS)
+		else if (fuse_stat) {
+		    Lf->nlink = fuse_links;
+		    Lf->nlink_def = 1;
+		}
+#endif	/* defined(HASFUSEFS) */
+
+#if	defined(HASMSDOSFS)
+		else if (msdos_stat) {
+		    Lf->nlink = msdos_links;
+		    Lf->nlink_def = 1;
+		}
+#endif	/* defined(HASMSDOSFS) */
+
 		else if (d) {
 		    Lf->nlink = d->de_links;
 		    Lf->nlink_def = 1;
 		}
-#endif	/* FREEBSDV>=5000 */
 
 		break;
 
@@ -1208,6 +1306,15 @@ process_overlaid_node:
 		break;
 #endif	/* defined(HASPSEUODOFS) */
 
+# if	defined(HAS_TMPFS)
+	    case N_TMP:
+		if (tnp) {
+		    Lf->nlink = (long)tnp->tn_links;
+		    Lf->nlink_def = 1;
+		}
+		break;
+# endif	/* defined(HAS_TMPFS) */
+
 	    }
 	    if (Lf->nlink_def && Nlink && (Lf->nlink < Nlink))
 		Lf->sf |= SELNLINK;
@@ -1266,8 +1373,8 @@ process_overlaid_node:
 	    ty = "FIFO";
 	    break;
 	default:
-	    (void) snpf(Lf->type, sizeof(Lf->type), "%04o", (type & 0xfff));
-	    ty = (char *)NULL;
+	     (void) snpf(Lf->type, sizeof(Lf->type), "%04o", (type & 0xfff));
+	     ty = (char *)NULL;
 	}
 	if (ty)
 	    (void) snpf(Lf->type, sizeof(Lf->type), "%s", ty);
@@ -1283,28 +1390,10 @@ process_overlaid_node:
 	if (type == VBAD)
 	    (void) snpf(Namech, Namechl, "(revoked)");
 
-#if	FREEBSDV<5000
-	else if (m) {
-	    Lf->dev_def = Lf->rdev_def = 0;
-	    (void) snpf(Namech, Namechl, "%#x", m->mfs_baseoff);
-	    (void) snpf(dev_ch, sizeof(dev_ch), "    memory");
-	    enter_dev_ch(dev_ch);
-	}
-#endif	/* FREEBSDV<5000 */
-
-
 #if	defined(HASPROCFS)
 	else if (p) {
 	    Lf->dev_def = Lf->rdev_def = 0;
 
-# if	FREEBSDV<2000
-	    if (type == VDIR)
-		(void) snpf(Namech, Namechl, "/%s", HASPROCFS);
-	    else
-		(void) snpf(Namech, Namechl, "/%s/%0*d", HASPROCFS, PNSIZ,
-		    p->pfs_pid);
-	    enter_nm(Namech);
-# else	/* FREEBSDV>=2000 */
 	    ty = (char *)NULL;
 	    (void) snpf(Namech, Namechl, "/%s", HASPROCFS);
 	    switch (p->pfs_type) {
@@ -1357,7 +1446,6 @@ process_overlaid_node:
 		ty = "PGID";
 		break;
 
-#  if	FREEBSDV>=3000
 	    case Pmap:
 		ep = endnm(&sz);
 		(void) snpf(ep, sz, "/%d/map", p->pfs_pid);
@@ -1368,14 +1456,12 @@ process_overlaid_node:
 		(void) snpf(ep, sz, "/%d/etype", p->pfs_pid);
 		ty = "PETY";
 		break;
-#  endif	/* FREEBSDV>=3000 */
 
 	    }
 	    if (ty)
 		(void) snpf(Lf->type, sizeof(Lf->type), "%s", ty);
 	    enter_nm(Namech);
 
-# endif	/* FREEBSDV<2000 */
 	}
 #endif	/* defined(HASPROCFS) */
 
@@ -1439,7 +1525,6 @@ process_overlaid_node:
 }
 
 
-#if	FREEBSDV>=2020
 /*
  * process_pipe() - process a file structure whose type is DTYPE_PIPE
  */
@@ -1492,4 +1577,58 @@ process_pipe(pa)
 	if (Namech[0])
 	    enter_nm(Namech);
 }
-#endif	/* FREEBSDV>=2020 */
+
+
+#if	defined(HASPTSFN) && defined(DTYPE_PTS)
+/*
+ * process_pts - process a file structure whose type is DTYPE_PTS
+ */
+
+void process_pts(tp)
+	KA_T tp;			/* f_data pointer to tty structure */
+{
+	dev_t dev;			/* IFCHR device number */
+	struct tty t;			/* tty structure */
+
+	(void) snpf(Lf->type, sizeof(Lf->type), "PTS");
+/*
+ * Read the tty structure.  Quit if it can't be read.
+ */
+	if (!tp || kread(tp, (char *)&t, sizeof(t))) {
+	    (void) snpf(Namech, Namechl,
+		"can't read DTYPE_PTS tty struct: %s",
+		print_kptr((KA_T)tp, (char *)NULL, 0));
+	    enter_nm(Namech);
+	    return;
+	}
+/*
+ * Convert the tty's cdev from kernel to user form.
+ *
+ * Set the device number to DevDev, the device number of /dev.
+ *
+ * Set the inode number to the device number.
+ *
+ * Set the file type to N_CHR for a character device (That's what a PTS is.)
+ *
+ * Force the use of offset from file structure.
+ *
+ * Set rdev to the converted device.
+ *
+ * Force the reloading of the device cache.
+ */
+	if ((dev = Dev2Udev((KA_T)t.t_dev)) == NODEV) {
+	    (void) snpf(Namech, Namechl,
+		"can't convert device in DTYPE_PTS tty struct: %s",
+		print_kptr((KA_T)tp, (char *)NULL, 0));
+	    enter_nm(Namech);
+	    return;
+	}
+	Lf->dev = DevDev;
+	Lf->inode = (INODETYPE)dev;
+	Lf->inp_ty = Lf->dev_def = Lf->rdev_def = 1;
+	Lf->ntype = N_CHR;
+	Lf->off_def = 1;
+	Lf->rdev = dev;
+	DCunsafe = 1;
+}
+#endif	/* defined(HASPTSFN) && defined(DTYPE_PTS) */
