@@ -35,7 +35,6 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 2008 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode2.c,v 1.6 2015/07/07 20:23:43 abe Exp $";
 #endif
 
 
@@ -49,9 +48,16 @@ static char *rcsid = "$Id: dnode2.c,v 1.6 2015/07/07 20:23:43 abe Exp $";
  *
  * Note: clang's complaint about VOP_FSYNC can't be avoided.
  */
-#define	VOP_UNLOCK(vp, f)	((void)0)
+#include <sys/param.h>
+#if __FreeBSD_version >= 1300074
+#define VOP_UNLOCK_FLAGS(vp, f) ((void)0)
+#define VOP_UNLOCK(vp)  ((void)0)
+#else
+#define VOP_UNLOCK(vp, f)       ((void)0)
+#endif
 # endif	/* defined(__clang__) */
 
+#define	KLD_MODULE		/* for ARM: prevent "ARM_NARCH is 0 " error */
 #include <sys/zfs_znode.h>
 #undef	_KERNEL
 
@@ -143,7 +149,7 @@ readzfsnode(za, zi, vr)
  * A gcc work-around
  */
 
-int     _cv_timedwait_sbt(struct cv *cvp, struct lock_object *lock,       
+int     _cv_timedwait_sbt(struct cv *cvp, struct lock_object *lock,
             sbintime_t sbt, sbintime_t pr, int flags)
 {
 	return(0);
