@@ -32,11 +32,18 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.19 2015/07/07 20:23:43 abe Exp $";
 #endif
 
 #include "lsof.h"
 
+/*
+ * This is not an exact version but it should not matter. At worst there
+ * is a small version window where this lsof does not compile on older
+ * -CURRENT.
+ */
+#if __FreeBSD_version >= 1300081
+#define HAS_PWD
+#endif
 
 _PROTOTYPE(static void enter_vn_text,(KA_T va, int *n));
 _PROTOTYPE(static void get_kernel_access,(void));
@@ -89,7 +96,7 @@ enter_vn_text(va, n)
 	    if (!Vp) {
 		(void) fprintf(stderr, "%s: no txt ptr space, PID %d\n",
 		    Pn, Lp->pid);
-		Exit(1);
+		Error();
 	    }
 	}
 /*
@@ -118,6 +125,9 @@ gather_proc_info()
 					 *	   streams with eXPORT data,
 					 *	   where supported */
 	struct filedesc fd;
+#if	defined(PWDDESC_KVM_LOAD_PWD)
+	struct pwddesc pd;
+#endif	/* defined(PWDDESC_KVM_LOAD_PWD) */
 	int i, nf;
 	MALLOC_S nb;
 
@@ -132,6 +142,15 @@ gather_proc_info()
 	KA_T fa;
 #endif	/* defined(HAS_FDESCENTTBL) */
 
+#if	defined(HAS_PWD)
+	struct pwd pwd;
+	KA_T pwd_addr;
+#endif	/* defined(HAS_FDESCENTTBL) */
+
+	struct vnode *cdir;
+	struct vnode *rdir;
+	struct vnode *jdir;
+
 	static ofb_t *ofb = NULL;
 	static int ofbb = 0;
 	int pgid, pid;
@@ -141,13 +160,7 @@ gather_proc_info()
 	int tid;			/* thread (task) ID */
 	uid_t uid;
 
-#if	FREEBSDV<2000
-	struct proc *p;
-	struct pcred pc;
-	struct pgrp pg;
-#else	/* FREEBSDV>=2000 */
 	struct kinfo_proc *p;
-#endif	/* FREEBSDV<2000 */
 
 #if	defined(HASFSTRUCT) && !defined(HAS_FILEDESCENT)
 	static char *pof = (char *)NULL;
@@ -190,7 +203,7 @@ gather_proc_info()
 	     * If only ORed process selection options have been specified,
 	     * enable conditional file skipping and socket file only checking.
 	     */
-		if ((Selflags & SELFILE) || !(Selflags & SELPROC))
+		if ((Selflags & SELFILE) || !(Selflags & SelProc))
 		    cckreg = ckscko = 0;
 		else
 		    cckreg = ckscko = 1;
@@ -207,10 +220,6 @@ gather_proc_info()
  * Read the process table.
  */
 
-#if	FREEBSDV<2000
-	if ((Np = kvm_getprocs(KINFO_PROC_ALL, 0)) < 0)
-#else	/* FREEBSDV>=2000 */
-
 # if	!defined(KERN_PROC_PROC)
 #define	KERN_PROC_PROC  KERN_PROC_ALL
 # endif	/* !defined(KERN_PROC_PROC) */
@@ -218,62 +227,28 @@ gather_proc_info()
 	if ((P = kvm_getprocs(Kd, Ftask ? KERN_PROC_ALL : KERN_PROC_PROC,
 			      0, &Np))
 	== NULL)
-#endif	/* FREEBSDV<2000 */
 
 	{
 	    (void) fprintf(stderr, "%s: can't read process table: %s\n",
 		Pn,
 
-#if	FREEBSDV<2000
-		kvm_geterr()
-#else	/* FREEBSDV>=2000 */
 		kvm_geterr(Kd)
-#endif	/* FREEBSDV<2000 */
 
 	    );
-	    Exit(1);
+	    Error();
 	}
 /*
  * Examine proc structures and their associated information.
  */
 
-#if	FREEBSDV<2000
-	for (px = 0; px < Np; px++)
-#else	/* FREEBSDV>=2000 */
 	for (p = P, px = 0; px < Np; p++, px++)
-#endif	/* FREEBSDV<2000 */
 
 	{
 
-#if	FREEBSDV<2000
-	/*
-	 * Read process information, process group structure (if
-	 * necessary), and User ID (if necessary).
-	 */
-	    if (!(p = kvm_nextproc()))
-		continue;
-	    if (p->P_STAT == 0 || p->P_STAT == SZOMB)
-		continue;
-	    pg.pg_id = 0;
-	    if (Fpgid && p->P_PGID) {
-		if (kread((KA_T)p->P_PGID, (char *)&pg, sizeof(pg)))
-		    continue;
-	    }
-	    pgid = pg.pg_id;
-	    if (!p->p_cred
-	    ||  kread((KA_T)p->p_cred, (char *)&pc, sizeof(pc)))
-		continue;
-	    uid = pc.p_ruid;
-#else	/* FREEBSDV>=2000 */
 	    if (p->P_STAT == 0 || p->P_STAT == SZOMB)
 		continue;
 	    pgid = p->P_PGID;
-# if	FREEBSDV<5000
-	    uid = p->kp_eproc.e_ucred.cr_uid;
-# else	/* FREEBSDV>=5000 */
 	    uid = p->ki_uid;
-# endif	/* FREEBSDV<5000 */
-#endif	/* FREEBSDV<2000 */
 
 #if	defined(HASPPID)
 	    ppid = p->P_PPID;
@@ -305,13 +280,33 @@ gather_proc_info()
 	    if (!fd.fd_files
 	    ||  kread((KA_T)fd.fd_files, (char *)&fdt, sizeof(fdt)))
 		continue;
-	    if (!fd.fd_refcnt || fd.fd_lastfile > fdt.fdt_nfiles)
+	    if (!fd.fd_refcnt)
 		continue;
 #else	/* !defined(HAS_FDESCENTTBL) */
 	    if (!fd.fd_refcnt || fd.fd_lastfile > fd.fd_nfiles)
 		continue;
 #endif	/* defined(HAS_FDESCENTTBL) */
 
+#if	defined(HAS_PWD)
+	    cdir = rdir = jdir = NULL;
+#if	  defined(PWDDESC_KVM_LOAD_PWD)
+	    pwd_addr = (KA_T)PWDDESC_KVM_LOAD_PWD(&pd);
+#else   /* defined(PWDDESC_KVM_LOAD_PWD) */
+	    pwd_addr = (KA_T)FILEDESC_KVM_LOAD_PWD(&fd);
+#endif  /* defined(PWDDESC_KVM_LOAD_PWD) */
+	    if (pwd_addr != 0) {
+		    if (!kread(pwd_addr, (char *)&pwd, sizeof(pwd))) {
+			    cdir = pwd.pwd_cdir;
+			    rdir = pwd.pwd_rdir;
+			    jdir = pwd.pwd_jdir;
+		    }
+	    }
+#else
+	    cdir = fd.fd_cdir;
+	    rdir = fd.fd_rdir;
+	    jdir = fd.fd_jdir;
+#endif
+
 	/*
 	 * Allocate a local process structure.
 	 */
@@ -324,7 +319,7 @@ gather_proc_info()
 	     * socket file only checking, based on the process' selection
 	     * status.
 	     */
-		ckscko = (sf & SELPROC) ? 0 : 1;
+		ckscko = (sf & SelProc) ? 0 : 1;
 	    }
 	    alloc_lproc(p->P_PID, pgid, ppid, (UID_ARG)uid, p->P_COMM,
 		(int)pss, (int)sf);
@@ -347,36 +342,34 @@ gather_proc_info()
 	/*
 	 * Save current working directory information.
 	 */
-	    if (!ckscko && fd.fd_cdir) {
+	    if (!ckscko && cdir) {
 		alloc_lfile(CWD, -1);
 		Cfp = (struct file *)NULL;
-		process_node((KA_T)fd.fd_cdir);
+		process_node((KA_T)cdir);
 		if (Lf->sf)
 		    link_lfile();
 	    }
 	/*
 	 * Save root directory information.
 	 */
-	    if (!ckscko && fd.fd_rdir) {
+	    if (!ckscko && rdir) {
 		alloc_lfile(RTD, -1);
 		Cfp = (struct file *)NULL;
-		process_node((KA_T)fd.fd_rdir);
+		process_node((KA_T)rdir);
 		if (Lf->sf)
 		    link_lfile();
 	    }
 
-#if	FREEBSDV>=5000
 	/*
 	 * Save jail directory information.
 	 */
-	    if (!ckscko && fd.fd_jdir) {
+	    if (!ckscko && jdir) {
 		alloc_lfile("jld", -1);
 		Cfp = (struct file *)NULL;
-		process_node((KA_T)fd.fd_jdir);
+		process_node((KA_T)jdir);
 		if (Lf->sf)
 		    link_lfile();
 	    }
-#endif	/* FREEBSDV>=5000 */
 
 	/*
 	 * Save information on the text file.
@@ -404,7 +397,7 @@ gather_proc_info()
 		if (!ofb) {
 		    (void) fprintf(stderr, "%s: PID %d, no file * space\n",
 			Pn, p->P_PID);
-		    Exit(1);
+		    Error();
 		}
 		ofbb = nb;
 	    }
@@ -431,7 +424,7 @@ gather_proc_info()
 		    if (!pof) {
 			(void) fprintf(stderr,
 			    "%s: PID %d, no file flag space\n", Pn, p->P_PID);
-			Exit(1);
+			Error();
 		    }
 		    pofb = nb;
 		}
@@ -504,7 +497,7 @@ get_kernel_access()
 	    if (!(Nmlst = get_nlist_path(1))) {
 		(void) fprintf(stderr,
 		    "%s: can't get kernel name list path\n", Pn);
-		Exit(1);
+		Error();
 	    }
 	}
 #endif	/* defined(N_UNIX) */
@@ -522,29 +515,21 @@ get_kernel_access()
  */
 	if ((Memory && !is_readable(Memory, 1))
 	||  (Nmlst && !is_readable(Nmlst, 1)))
-	    Exit(1);
+	    Error();
 #endif	/* defined(WILLDROPGID) */
 
 /*
  * Open kernel memory access.
  */
 
-#if	FREEBSDV<2000
-	if (kvm_openfiles(Nmlst, Memory, NULL) == -1)
-#else	/* FREEBSDV>=2000 */
 	if ((Kd = kvm_open(Nmlst, Memory, NULL, O_RDONLY, NULL)) == NULL)
-#endif	/* FREEBSDV<2000 */
 
 	{
 	    (void) fprintf(stderr,
 		"%s: kvm_open%s(execfile=%s, corefile=%s): %s\n",
 		Pn,
 
-#if	FREEBSDV<2000
-		"files",
-#else	/* FREEBSDV>=2000 */
 		"",
-#endif	/* FREEBSDV<2000 */
 
 		Nmlst ? Nmlst : "default",
 		Memory ? Memory :
@@ -556,15 +541,26 @@ get_kernel_access()
 #endif	/* defined(_PATH_MEM) */
 
 		strerror(errno));
-	    Exit(1);
+	    Error();
 	}
 	(void) build_Nl(Drive_Nl);
 	if (kvm_nlist(Kd, Nl) < 0) {
 	    (void) fprintf(stderr, "%s: can't read namelist from %s\n",
 		Pn, Nmlst);
-	    Exit(1);
+	    Error();
 	}
 
+#if	defined(X_BADFILEOPS)
+/*
+ * Get kernel's badfileops address (for process_file()).
+ */
+	if (get_Nl_value(X_BADFILEOPS, (struct drive_Nl *)NULL, &X_bfopsa) < 0
+	||  !X_bfopsa)
+	{
+	    X_bfopsa = (KA_T)0;
+	}
+#endif	/* defined(X_BADFILEOPS) */
+
 #if	defined(WILLDROPGID)
 /*
  * Drop setgid permission, if necessary.
@@ -602,7 +598,7 @@ get_nlist_path(ap)
 		(void) fprintf(stderr,
 		    "%s: can't allocate %d bytes for boot file path: %s\n",
 		    Pn, (int)bfl, bf);
-		Exit(1);
+		Error();
 	    }
 	    (void) snpf(bfc, bfl, "%s", bf);
 	    return(bfc);
@@ -635,15 +631,33 @@ kread(addr, buf, len)
 {
 	int br;
 
-#if	FREEBSDV<2000
-	br = kvm_read((void *)addr, (void *)buf, len);
-#else	/* FREEBSDV>=2000 */
 	br = kvm_read(Kd, (u_long)addr, buf, len);
-#endif	/* FREEBSDV<2000 */
 
 	return((br == len) ? 0 : 1);
 }
 
+static int
+vm_map_reader(void *token, vm_map_entry_t addr, vm_map_entry_t dest)
+{
+	return (kread((KA_T)addr, (char *)dest, sizeof(*dest)));
+}
+
+#if	__FreeBSD_version < 1300060
+typedef int vm_map_entry_reader(void *token, vm_map_entry_t addr,
+    vm_map_entry_t dest);
+
+static inline vm_map_entry_t
+vm_map_entry_read_succ(void *token, struct vm_map_entry *const clone,
+     vm_map_entry_reader reader)
+{
+	vm_map_entry_t next;
+
+	next = clone->next;
+	if (!reader(token, next, clone))
+		return (NULL);
+	return (next);
+}
+#endif	/*  __FreeBSD_version < 1300060 */
 
 /*
  * process_text() - process text information
@@ -659,10 +673,6 @@ process_text(vm)
 	struct vm_object vmo;
 	struct vmspace vmsp;
 
-#if	FREEBSDV<2020
-	struct pager_struct pg;
-#endif	/* FREEBSDV<2020 */
-
 /*
  * Read the vmspace structure for the process.
  */
@@ -671,20 +681,15 @@ process_text(vm)
 /*
  * Read the vm_map structure.  Search its vm_map_entry structure list.
  */
+	vmme = vmsp.vm_map.header;
+	e = &vmme;
 	for (i = 0; i < vmsp.vm_map.nentries; i++) {
 
 	/*
 	 * Read the next vm_map_entry.
 	 */
-	    if (i == 0)
-		e = &vmsp.vm_map.header;
-	    else {
-		if (!(ka = (KA_T)e->next))
-		    return;
-		e = &vmme;
-		if (kread(ka, (char *)e, sizeof(vmme)))
-		    return;
-	    }
+	    if (!vm_map_entry_read_succ(NULL, e, vm_map_reader))
+		return;
 
 #if	defined(MAP_ENTRY_IS_A_MAP)
 	    if (e->eflags & (MAP_ENTRY_IS_A_MAP|MAP_ENTRY_IS_SUB_MAP))
@@ -702,29 +707,16 @@ process_text(vm)
 		 j < 2 && ka;
 		 j++,
 
-#if	FREEBSDV<2020
-		 ka = (KA_T)vmo.shadow
-#else	/* FREEBSDV>=2020 */
 		 ka = (KA_T)vmo.backing_object
-#endif	/* FREEBSDV<2020 */
 		 )
 	    {
 		if (kread(ka, (char *)&vmo, sizeof(vmo)))
 		    break;
 
-#if	FREEBSDV<2020
-		if ((ka = (KA_T)vmo.pager) == NULL
-		||  kread(ka, (char *)&pg, sizeof(pg)))
-		    continue;
-		if (pg.pg_handle == NULL || pg.pg_type != PG_VNODE)
-		    continue;
-		(void) (enter_vn_text((KA_T)pg.pg_handle, &n));
-#else	/* FREEBSDV>=2020 */
 		if (vmo.type != OBJT_VNODE
 		||  vmo.handle == (void *)NULL)
 		    continue;
 		(void) (enter_vn_text((KA_T)vmo.handle, &n));
-#endif	/* FREEBSDV<2020 */
 
 	    }
 	}
