@@ -33,17 +33,17 @@
 
 
 /*
- * $Id: dproto.h,v 1.14 2009/03/25 19:23:06 abe Exp $
+ * $Id: dproto.h,v 1.15 2018/02/14 14:26:03 abe Exp $
  */
 
 
-#if	FREEBSDV>=5000 && defined(HAS_NO_SI_UDEV)
+#if	defined(HAS_NO_SI_UDEV)
 # if	defined(HAS_CONF_MINOR)|| defined(HAS_CDEV2PRIV)
 _PROTOTYPE(extern dev_t Dev2Udev,(KA_T c));
 # else	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
 _PROTOTYPE(extern dev_t Dev2Udev,(struct cdev *c));
 # endif	/* defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV) */
-#endif	/* FREEBSDV>=5000 && defined(HAS_NO_SI_UDEV) */
+#endif	/* defined(HAS_NO_SI_UDEV) */
 
 #if	!defined(N_UNIX)
 _PROTOTYPE(extern char *get_nlist_path,(int ap));
@@ -53,14 +53,24 @@ _PROTOTYPE(extern int is_file_named,(char *p, int cd));
 _PROTOTYPE(extern void process_socket,(KA_T sa));
 _PROTOTYPE(extern struct l_vfs *readvfs,(KA_T vm));
 
+#if	defined(HASPTSFN)
+_PROTOTYPE(extern void process_pts,(KA_T ta));
+#endif	/* defined(HASPTSFN) */
+
 #if	defined(HASKQUEUE)
 _PROTOTYPE(extern void process_kqueue,(KA_T ka));
 #endif	/* defined(HASKQUEUE) */
 
-#if	FREEBSDV>=2020
 _PROTOTYPE(extern void process_pipe,(KA_T pa));
-#endif	/* FREEBSDV>=2020 */
+
+#if	defined(HASFUSEFS)
+_PROTOTYPE(extern int read_fuse_node,(struct vnode *v, dev_t *d, int *dd, INODETYPE *ino, long *nl, SZOFFTYPE *sz));
+#endif	/* defined(HASFUSEFS) */
 
 #if	defined(HAS9660FS)
 _PROTOTYPE(extern int read_iso_node,(struct vnode *v, dev_t *d, int *dd, INODETYPE *ino, long *nl, SZOFFTYPE *sz));
 #endif	/* defined(HAS9660FS) */
+
+#if	defined(HASMSDOSFS)
+_PROTOTYPE(extern int read_msdos_node,(struct vnode *v, dev_t *d, int *dd, INODETYPE *ino, long *nl, SZOFFTYPE *sz));
+#endif	/* defined(HASMSDOSFS) */
