@@ -32,7 +32,6 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dstore.c,v 1.7 2004/07/06 19:18:57 abe Exp $";
 #endif
 
 
@@ -48,29 +47,17 @@ struct file *Cfp;			/* curent file's file struct pointer */
 
 struct drive_Nl Drive_Nl[] = {
 
-#if	FREEBSDV<2005
-	{ X_NCACHE,	"_nchhead"	},
-#else	/* FREEBSDV>=2005 */
-# if	FREEBSDV<2010
-	{ X_NCACHE,	"_nclruhead"	},
-# else	/* FREEBSDV>=2010 */
+	{ X_BADFILEOPS,	"badfileops"	},
+
 	{ X_NCACHE,	"_nchashtbl"	},
-# endif	/* FREEBSDV<2010 */
-#endif	/* FREEBSDV<2005 */
 
-#if	FREEBSDV<2010
-	{ X_NCSIZE,	"_numcache"	},
-#else	/* FREEBSDV>=2010 */
 	{ X_NCSIZE,	"_nchash"	},
-#endif	/* FREEBSDV<2010 */
 
 	{ "",		""		},
 	{ NULL,		NULL		}
 };
 
-#if	FREEBSDV>=2000
 kvm_t *Kd = NULL;			/* kvm descriptor */
-#endif	/* FREEBSDV>=2000 */
 
 #if	defined(P_ADDR)
 KA_T Kpa;				/* kernel proc struct address */
@@ -80,9 +67,7 @@ struct l_vfs *Lvfs = NULL;		/* local vfs structure table */
 
 int Np = 0;				/* number of kernel processes */
 
-#if	FREEBSDV>=2000
 struct kinfo_proc *P = NULL;		/* local process table copy */
-#endif	/* FREEBSDV>=2000 */
 
 #if	defined(HASFSTRUCT)
 /*
@@ -97,8 +82,15 @@ struct pff_tab Pff_tab[] = {
 	{ (long)FAPPEND,	FF_APPEND	},
 	{ (long)FASYNC,		FF_ASYNC	},
 	{ (long)FFSYNC,		FF_FSYNC	},
+
+# if	defined(FMARK)
 	{ (long)FMARK,		FF_MARK		},
+# endif	/* defined(FMARK) */
+
+# if	defined(FDEFER)
 	{ (long)FDEFER,		FF_DEFER	},
+# endif	/* defined(FDEFER) */
+
 	{ (long)FHASLOCK,	FF_HASLOCK	},
 	{ (long)O_NOCTTY,	FF_NOCTTY	},
 	{ (long)0,		NULL 		}
@@ -122,3 +114,10 @@ struct pff_tab Pof_tab[] = {
 	{ (long)0,		NULL		}
 };
 #endif	/* defined(HASFSTRUCT) */
+
+
+/*
+ * Kernel's bad file operations address
+ */
+
+KA_T X_bfopsa;				/* badfileops kernel address */
