@@ -32,7 +32,6 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.15 2001/06/19 13:29:19 abe Exp $";
 #endif
 
 #if	defined(HPUXKERNBITS) && HPUXKERNBITS>=64
@@ -53,6 +52,15 @@ typedef int time_t;
 #endif	/* HPUXV>=800 && defined(HPUX_CCITT) */
 
 
+/*
+ * Local definitions
+ */
+
+#if	defined(HASTCPOPT)
+#define	TF_NODELAY	0x1		/* TCP_NODELAY (Nagle algorithm) */
+#endif	/* defined(HASTCPOPT) */
+
+
 #if	HPUXV>=1030
 /*
  * print_tcptpi() - print TCP/TPI info
@@ -214,6 +222,152 @@ print_tcptpi(nl)
 	}
 # endif	/* defined(HASTCPTPIQ) */
 
+#if	defined(HASSOOPT)
+	if (Ftcptpi & TCPTPI_FLAGS) {
+	    int opt;
+
+	    if ((opt = Lf->lts.opt)
+	    ||  Lf->lts.qlens || Lf->lts.qlims || Lf->lts.rbszs || Lf->lts.sbsz
+	    ) {
+		char sep = ' ';
+
+		if (Ffield)
+		    sep = LSOF_FID_TCPTPI;
+		else if (!ps)
+		    sep = '(';
+		(void) printf("%cSO", sep);
+		ps++;
+		sep = '=';
+
+# if	defined(SO_BROADCAST)
+		if (opt & SO_BROADCAST) {
+		    (void) printf("%cBROADCAST", sep);
+		    opt &= ~SO_BROADCAST;
+		    sep = ',';
+		}
+# endif	/* defined(SO_BROADCAST) */
+
+# if	defined(SO_DEBUG)
+		if (opt & SO_DEBUG) {
+		    (void) printf("%cDEBUG", sep);
+		    opt &= ~ SO_DEBUG;
+		    sep = ',';
+		}
+# endif	/* defined(SO_DEBUG) */
+
+# if	defined(SO_DONTROUTE)
+		if (opt & SO_DONTROUTE) {
+		    (void) printf("%cDONTROUTE", sep);
+		    opt &= ~SO_DONTROUTE;
+		    sep = ',';
+		}
+# endif	/* defined(SO_DONTROUTE) */
+
+# if	defined(SO_KEEPALIVE)
+		if (opt & SO_KEEPALIVE) {
+		    (void) printf("%cKEEPALIVE", sep);
+		    if (Lf->lts.kai)
+			(void) printf("=%d", Lf->lts.kai);
+		    opt &= ~SO_KEEPALIVE;
+		    sep = ',';
+		}
+# endif	/* defined(SO_KEEPALIVE) */
+
+# if	defined(SO_LINGER)
+		if (opt & SO_LINGER) {
+		    (void) printf("%cLINGER", sep);
+		    if (Lf->lts.ltm)
+			(void) printf("=%d", Lf->lts.ltm);
+		    opt &= ~SO_LINGER;
+		    sep = ',';
+		}
+# endif	/* defined(SO_LINGER) */
+
+# if	defined(SO_OOBINLINE)
+		if (opt & SO_OOBINLINE) {
+		    (void) printf("%cOOBINLINE", sep);
+		    opt &= ~SO_OOBINLINE;
+		    sep = ',';
+		}
+# endif	/* defined(SO_OOBINLINE) */
+
+		if (Lf->lts.qlens) {
+		    (void) printf("%cQLEN=%u", sep, Lf->lts.qlen);
+		    sep = ',';
+		}
+		if (Lf->lts.qlims) {
+		    (void) printf("%cQLIM=%u", sep, Lf->lts.qlim);
+		    sep = ',';
+		}
+
+# if	defined(SO_REUSEADDR)
+		if (opt & SO_REUSEADDR) {
+		    (void) printf("%cREUSEADDR", sep);
+		    opt &= ~SO_REUSEADDR;
+		    sep = ',';
+		}
+# endif	/* defined(SO_REUSEADDR) */
+
+# if	defined(SO_REUSEPORT)
+		if (opt & SO_REUSEPORT) {
+		    (void) printf("%cREUSEPORT", sep);
+		    opt &= ~SO_REUSEPORT;
+		    sep = ',';
+		}
+# endif	/* defined(SO_REUSEPORT) */
+
+# if	defined(SO_USELOOPBACK)
+		if (opt & SO_USELOOPBACK) {
+		    (void) printf("%cUSELOOPBACK", sep);
+		    opt &= ~SO_USELOOPBACK;
+		    sep = ',';
+		}
+# endif	/* defined(SO_USELOOPBACK) */
+
+		if (opt)
+		    (void) printf("%cUNKNOWN=%#x", sep, opt);
+		if (Ffield)
+		    putchar(Terminator);
+	    }
+	}
+#endif	/* defined(HASSOOPT) */
+
+#if	defined(HASTCPOPT)
+	if (Ftcptpi & TCPTPI_FLAGS) {
+	    int topt;
+
+	    if ((topt = Lf->lts.topt) || Lf->lts.msss) {
+		char sep = ' ';
+
+		if (Ffield)
+		    sep = LSOF_FID_TCPTPI;
+		else if (!ps)
+		    sep = '(';
+		(void) printf("%cTF", sep);
+		ps++;
+		sep = '=';
+
+		if (Lf->lts.msss) {
+		    (void) printf("%cMSS=%lu", sep, Lf->lts.mss);
+		    sep = ',';
+		}
+
+#  if	defined(TF_NODELAY)
+		if (topt & TF_NODELAY) {
+		    (void) printf("%cNODELAY", sep);
+		    topt &= ~TF_NODELAY;
+		    sep = ',';
+		}
+#  endif	/* defined(TF_NODELAY) */
+
+		if (topt)
+		    (void) printf("%cUNKNOWN=%#x", sep, topt);
+		if (Ffield)
+		    putchar(Terminator);
+	    }
+	}
+# endif	/* defined(HASTCPOPT) */
+
 # if	defined(HASTCPTPIW)
 	if (Ftcptpi & TCPTPI_WINDOWS) {
 	    if (Lf->lts.rws) {
@@ -689,7 +843,7 @@ process_socket(sa)
 	 * above.
 	 */
 	    if (unp.unp_ino) {
-		Lf->inode = (unsigned long)unp.unp_ino;
+		Lf->inode = (INODETYPE)unp.unp_ino;
 		Lf->inp_ty = 1;
 	    }
 	    ua = (struct sockaddr_un *)NULL;
@@ -710,7 +864,7 @@ process_socket(sa)
 			    (void) fprintf(stderr,
 				"%s: no space (%d) for UNIX socket address\n",
 				Pn, dbl);
-			    Exit(1);
+			    Error();
 			}
 		    }
 		    if (kread((KA_T)db.db_base, dbf, db.db_size) == 0) {
@@ -890,7 +1044,7 @@ process_stream_sock(ip, pcb, pn, vt)
 
 	    if ((rq = (int)tc.tcp_rnxt - (int)tc.tcp_rack - 1) < 0)
 		rq = 0;
-	    if ((sq = (int)tc.tcp_snxt - (int)tc.tcp_suna) < 0)
+	    if ((sq = (int)tc.tcp_snxt - (int)tc.tcp_suna - 1) < 0)
 		sq  = 0;
 
 #  if	defined(HASTCPTPIQ)
@@ -915,6 +1069,56 @@ process_stream_sock(ip, pcb, pn, vt)
 	        Lf->off_def = 1;
 # endif	/* defined(HASTCPTPIQ) || defined(HASTCPTPIW) */
 
+# if	defined(HASTCPOPT)
+	    if (Ftcptpi & TCPTPI_FLAGS) {
+
+	    /*
+	     * Save TCP options and values..
+	     */
+		if (tc.tcp_naglim == (uint)1)
+		    Lf->lts.topt |= TF_NODELAY;
+		Lf->lts.mss = (unsigned long)tc.tcp_mss;
+		Lf->lts.msss = (unsigned char)1;
+	    }
+# endif	/* defined(HASTCPOPT) */
+
+# if	defined(HASSOOPT)
+	    if (Ftcptpi & TCPTPI_FLAGS) {
+
+	    /*
+	     * Save socket options.
+	     */
+		if (tc.tcp_broadcast)
+		    Lf->lts.opt |= SO_BROADCAST;
+		if (tc.tcp_so_debug)
+		    Lf->lts.opt |= SO_DEBUG;
+		if (tc.tcp_dontroute)
+		    Lf->lts.opt |= SO_DONTROUTE;
+		if (tc.tcp_keepalive_intrvl
+		&&  (tc.tcp_keepalive_intrvl != 7200000)
+		) {
+		    Lf->lts.opt |= SO_KEEPALIVE;
+		    Lf->lts.kai = (unsigned int)tc.tcp_keepalive_intrvl;
+		}
+		if (tc.tcp_lingering) {
+		    Lf->lts.opt |= SO_LINGER;
+		    Lf->lts.ltm = (unsigned int)tc.tcp_linger;
+		}
+		if (tc.tcp_oobinline)
+		    Lf->lts.opt |= SO_OOBINLINE;
+		if (tc.tcp_reuseaddr)
+		    Lf->lts.opt |= SO_REUSEADDR;
+		if (tc.tcp_reuseport)
+		    Lf->lts.opt |= SO_REUSEPORT;
+		if (tc.tcp_useloopback)
+		    Lf->lts.opt |= SO_USELOOPBACK;
+		Lf->lts.qlen = (unsigned int)tc.tcp_conn_ind_cnt;
+		Lf->lts.qlim = (unsigned int)tc.tcp_conn_ind_max;
+		if (Lf->lts.qlen || Lf->lts.qlim)
+		    Lf->lts.qlens = Lf->lts.qlims = (unsigned char)1;
+	    }
+# endif	/* defined(HASSOOPT) */
+
 	    Namech[0] = '\0';
 	    return;
 	} else if (pn && !strcmp(pn, "UDP")) {
