@@ -32,7 +32,6 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1999 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id";
 #endif
 
 
@@ -100,7 +99,7 @@ static int Nzpss = 0;			/* Nzps status: 1 = zeroed */
 
 
 /*
- * Local macroes
+ * Local macros
  */
 
 #define	HASHFSID(i)	(Ncfsid + \
@@ -207,7 +206,7 @@ ncache_alloc()
 	{
 	    (void) fprintf(stderr,
 		"%s: can't allocate %d local name cache entries\n", Pn, Nceh);
-	    Exit(1);
+	    Error();
 	}
 	if (Ncfsid)
 	    return;
@@ -216,7 +215,7 @@ ncache_alloc()
 	    (void) fprintf(stderr,
 		"%s: can't allocate %d local file system cache entries\n",
 		Pn, NFSIDH);
-	    Exit(1);
+	    Error();
 	}
 }
 
@@ -329,7 +328,7 @@ ncache_loadfs(fsid, fh)
  */
 	if (!(f = (struct l_fic *)malloc(sizeof(struct l_fic)))) {
 	    (void) fprintf(stderr, "%s: no fsid structure space\n", Pn);
-	    Exit(1);
+	    Error();
 	}
 	f->fsid = *fsid;
 	f->nc = 0;
@@ -356,13 +355,13 @@ ncache_loadfs(fsid, fh)
 		    (void) fprintf(stderr,
 			"%s: no name entry space (%d) for:%s\n",
 			Pn, nl + 1, mp[i].psr_name);
-		    Exit(1);
+		    Error();
 		}
 		if (!(nn = (struct l_nc *)malloc(sizeof(struct l_nc)))) {
 		    (void) fprintf(stderr,
 			"%s: no name cache entry space (%d) for: %s\n",
 			Pn, (int)sizeof(struct l_nc), mp[i].psr_name);
-		    Exit(1);
+		    Error();
 		}
 	    /*
 	     * Fill in name cache entry, complete with name and name length.
@@ -607,7 +606,7 @@ ncache_size()
 
 	if (pstat_getdynamic(&pd, sizeof(pd), 1, 0) != 1) {
 	    (void) fprintf(stderr, "%s: can't get dynamic status\n", Pn);
-	    Exit(1);
+	    Error();
 	}
 	Ndnlc = (int)pd.psd_dnlc_size;
 	for (Nceh = 1; Nceh < (Ndnlc + Ndnlc); Nceh <<= 1)
