@@ -1,5 +1,5 @@
 /*
- * dproc.c - NEXTSTEP and OpenStep process access functions for lsof
+ * dproc.c - NEXTSTEP and OPENSTEP process access functions for lsof
  */
 
 
@@ -32,7 +32,6 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.11 2000/11/03 18:05:31 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -83,7 +82,7 @@ ckkv(d, er, ev, ea)
 	if ((kr = host_kernel_version(host_self(), kv)) != KERN_SUCCESS) {
 	    (void) snpf(m, sizeof(m), "%s: can't get kernel version:", Pn);
 	    mach_error(m, kr);
-	    Exit(1);
+	    Error();
 	}
 /*
  * Skip blank-separated tokens until reaching "Mach".  The kernel version
@@ -129,21 +128,9 @@ gather_proc_info()
 	MALLOC_S nb;
 	short pss, sf;
 	struct task {			/* (Should come from <kern/task.h>.) */
-
-#if	defined(hppa)
-		caddr_t d1[6];
-#else	/* !defined(hppa) */
-		caddr_t d1[3];
-#endif	/* defined(hppa) */
-
+		caddr_t d1[SIMPLE_LOCK_SIZE + 2];
 		caddr_t map;
-
-#if	defined(hppa)
-		caddr_t d2[13];
-#else	/* !defined(hppa) */
-		caddr_t d2[10];
-#endif	/* defined(hppa) */
-
+		caddr_t d2[SIMPLE_LOCK_SIZE + 9];
 		struct utask *u_address;
 		struct proc *proc;
 	} t;
@@ -235,7 +222,7 @@ gather_proc_info()
 		    uf = (struct file **)realloc((MALLOC_P *)uf, nb);
 		if (!uf) {
 		    (void) fprintf(stderr, "%s: no uu_ofile space\n", Pn);
-		    Exit(1);
+		    Error();
 		}
 		ufb = nb;
 	    }
@@ -252,7 +239,7 @@ gather_proc_info()
 			pof = (char *)realloc((MALLOC_P *)pof, nb);
 		    if (!pof) {
 			(void) fprintf(stderr, "%s: no uu_pofile space\n", Pn);
-			Exit(1);
+			Error();
 		    }
 		    pofb = nb;
 		}
@@ -317,7 +304,7 @@ get_kernel_access()
  * See if the non-KMEM memory file is readable.
  */
 	if (Memory && !is_readable(Memory, 1))
-		Exit(1);
+		Error();
 #endif	/* defined(WILLDROPGID) */
 
 /*
@@ -326,7 +313,7 @@ get_kernel_access()
 	if ((Kd = open(Memory ? Memory : KMEM, O_RDONLY, 0)) < 0) {
 		(void) fprintf(stderr, "%s: can't open %s: %s\n", Pn,
 			Memory ? Memory : KMEM, strerror(errno));
-		Exit(1);
+		Error();
 	}
 
 #if	defined(WILLDROPGID)
@@ -340,7 +327,7 @@ get_kernel_access()
  * See if the name list file is readable.  Build Nl.
  */
 	if (Nmlst && !is_readable(Nmlst, 1))
-		Exit(1);
+		Error();
 #endif	/* defined(WILLDROPGID) */
 
 	(void) build_Nl(Drive_Nl);
@@ -356,7 +343,7 @@ get_kernel_access()
 		if (!(nl = (struct nlist *)malloc(Nll))) {
 			(void) fprintf(stderr,
 				"%s: no space (%d) for Nl[] copy\n", Pn, Nll);
-			Exit(1);
+			Error();
 		}
 		(void) bcopy((char *)Nl, (char *)nl, Nll);
 	}
@@ -368,12 +355,12 @@ get_kernel_access()
 	if (nlist(Nmlst ? Nmlst : VMUNIX, Nl) < 0) {
 		(void) fprintf(stderr, "%s: can't read namelist from %s\n",
 			Pn, Nmlst ? Nmlst : VMUNIX);
-                Exit(1);
+                Error();
 	}
 	if (get_Nl_value("aproc", Drive_Nl, &lv) < 0 || !lv) {
 		(void) fprintf(stderr, "%s: can't get proc table address\n",
 			Pn);
-		Exit(1);
+		Error();
 	}
 
 #if	defined(HAS_AFS)
@@ -385,7 +372,7 @@ get_kernel_access()
 	 * the AFS module name list file.  Make sure that other symbols that
 	 * appear in both name list files have the same values.
 	 */
-		if (get_Nl_value("arFID", Drive_Nl, &v[0]) >= 0 
+		if (get_Nl_value("arFID", Drive_Nl, &v[0]) >= 0
 		&&  get_Nl_value("avol",  Drive_Nl, &v[1]) >= 0
 		&&  get_Nl_value("avol",  Drive_Nl, &v[2]) >= 0
 		&&  (!vo[0] || !v[1] || !v[2]))
@@ -442,7 +429,12 @@ static void
 process_map(map)
 	caddr_t map;
 {
-	int i, j, n;
+	int i, j, n, ne;
+
+#if	STEPV<40
+/*
+ * Structures for NeXTSTEP and OPENSTEP < 4.0.
+ */
 	struct vm_map_entry {	/* (Should come from <vm/vm_map.h>). */
 		struct vm_map_entry *prev;
 		struct vm_map_entry *next;
@@ -460,14 +452,11 @@ process_map(map)
 		int inheritance;
 		int wired_count;
 	} vme, *vmep;
+
+#define VME_NEXT(entry) entry.next
+
 	struct vm_map {		/* (Should come from <vm/vm_map.h>.) */
-
-#if	defined(hppa)
-		caddr_t d1[6];
-#else	/* !defined(hppa) */
-		caddr_t d1[3];
-#endif	/* defined(hppa) */
-
+		caddr_t d1[SIMPLE_LOCK_SIZE + 2];
 		struct vm_map_entry header;
 		int nentries;
 		caddr_t pmap;
@@ -475,13 +464,7 @@ process_map(map)
 		boolean_t is_main_map;
 	} vmm;
 	struct vm_object {	/* (Should come from <vm/vm_object.h>.) */
-
-#if	defined(hppa)
-		caddr_t d1[8];
-#else	/* !defined(hppa) */
-		caddr_t d1[5];
-#endif	/* defined(hppa) */
-
+		caddr_t d1[SIMPLE_LOCK_SIZE + 4];
 		unsigned int size;
 		short ref_count, resident_page_count;
 		caddr_t copy;
@@ -490,6 +473,64 @@ process_map(map)
 		unsigned int paging_offset;
 		caddr_t shadow;
 	} vmo, vmso;
+#else	/* STEPV>=40 */
+/*
+ * Structures for OPENSTEP >= 4.0.
+ */
+	struct vm_map_links {   /* (Should come from <vm/vm_map.h>). */
+		struct vm_map_entry *prev;
+		struct vm_map_entry *next;
+		unsigned int start;
+		unsigned int end;
+	};
+	struct vm_map_entry {	/* (Should come from <vm/vm_map.h>). */
+		struct vm_map_links links;
+		caddr_t object;
+		unsigned int offset;
+		unsigned int
+			is_shared:1,
+			is_sub_map:1,
+			in_transition:1,
+			needs_wakeup:1,
+			behavior:2,
+			needs_copy:1,
+			protection:3,
+			max_protection:3,
+			inheritance:2,
+			pad1:1,
+			alias:8;
+		unsigned short wired_count;
+		unsigned short user_wired_count;
+	} vme, *vmep;
+
+#define VME_NEXT(entry) entry.links.next
+
+	struct vm_map_header {   /* (Should come from <vm/vm_map.h>.) */
+		struct vm_map_links links;
+		int nentries;
+		int entries_pageable;
+	};
+	struct vm_map {		     /* (Should come from <vm/vm_map.h>.) */
+		caddr_t d1[SIMPLE_LOCK_SIZE + 2];
+		struct vm_map_header hdr;
+		caddr_t pmap;
+		unsigned int size;
+		boolean_t is_main_map; /* Darwin header has this as ref_count,
+					* but we'll take some liberties ... */
+	} vmm;
+	struct vm_object {	     /* (Should come from <vm/vm_object.h>.) */
+		caddr_t d1[SIMPLE_LOCK_SIZE + 4];
+		unsigned int size;
+		short ref_count, resident_page_count;
+		caddr_t copy;
+		caddr_t shadow;
+		unsigned int shadow_offset;
+		caddr_t pager;
+		unsigned int paging_offset;
+		int pager_request;
+	} vmo, vmso;
+#endif	/* STEPV<40 */
+
 	struct vstruct {	/* (Should come from <vm/vnode_pager.h>.) */
 		boolean_t is_device;
 		caddr_t vs_pf;
@@ -511,68 +552,84 @@ process_map(map)
  * Look for non-map and non-sub-map vm map entries that have an object
  * with a shadow whose pager pointer addresses a non-swap-file istruct
  * that has a vnode pointer.  Process the unique vnodes found.
- */ 
-	for (i = n = 0, vme = vmm.header; i < vmm.nentries; i++) {
-		if (i) {
-			if (!vme.next
-			||  kread((KA_T)vme.next, (char *)&vme, sizeof(vme)))
-				continue;
-		}
-		if (vme.is_a_map || vme.is_sub_map)
-			continue;
-		if (!vme.object
-		||  kread((KA_T)vme.object, (char *)&vmo, sizeof(vmo)))
-			continue;
-		if (!vmo.shadow
-		||  kread((KA_T)vmo.shadow, (char *)&vmso, sizeof(vmso)))
-			continue;
-		if (!vmso.pager
-		||  kread((KA_T)vmso.pager, (char *)&vmp, sizeof(vmp)))
-			continue;
-		if (vmp.is_device || vmp.vs_swapfile || !vmp.vs_vp)
-			continue;
+ */
+#if	STEPV<40
+	vme = vmm.header;
+	ne = vmm.nentries;
+#else	/* STEPV>=40 */
+	if (!vmm.hdr.links.next
+	||  kread((KA_T)vmm.hdr.links.next, (char *)&vme, sizeof(vme)))
+	    return;
+	ne = vmm.hdr.nentries;
+#endif	/* STEPV<40 */
+
+	if (ne > 1000)
+	    ne = 1000;
+	for (i = n = 0; i < ne; i++) {
+	    if (i) {
+		if (!VME_NEXT(vme)
+		||  kread((KA_T)VME_NEXT(vme), (char *)&vme, sizeof(vme)))
+		    continue;
+	    }
+
+#if	STEPV<40
+	    if (vme.is_a_map || vme.is_sub_map)
+#else	/* STEPV>=40 */
+	    if (vme.is_sub_map)
+#endif	/* STEPV<40 */
+
+		continue;
+	    if (!vme.object
+	    ||  kread((KA_T)vme.object, (char *)&vmo, sizeof(vmo)))
+		continue;
+	    if (!vmo.shadow
+	    ||  kread((KA_T)vmo.shadow, (char *)&vmso, sizeof(vmso)))
+		continue;
+	    if (!vmso.pager
+	    ||  kread((KA_T)vmso.pager, (char *)&vmp, sizeof(vmp)))
+		continue;
+	    if (vmp.is_device || vmp.vs_swapfile || !vmp.vs_vp)
+		continue;
 	/*
 	 * See if the vnode has been processed before.
 	 */
-		for (j = 0; j < n; j++) {
-			if ((KA_T)vmp.vs_vp == Vp[j])
-				break;
-		}
-		if (j < n)
-			continue;
+	    for (j = 0; j < n; j++) {
+		if ((KA_T)vmp.vs_vp == Vp[j])
+		    break;
+	    }
+	    if (j < n)
+		continue;
 	/*
 	 * Process a new vnode.
 	 */
-		alloc_lfile("txt", -1);
-		FILEPTR = (struct file *)NULL;
-		process_node((KA_T)vmp.vs_vp);
-		if (Lf->sf)
-			link_lfile();
+	    alloc_lfile("txt", -1);
+	    FILEPTR = (struct file *)NULL;
+	    process_node((KA_T)vmp.vs_vp);
+	    if (Lf->sf)
+		link_lfile();
 	/*
 	 * Allocate space for remembering the vnode.
 	 */
-		if (!Vp) {
-			if (!(Vp = (KA_T *)malloc((MALLOC_S)
-			    (sizeof(struct vnode *) * 10))))
-			{
-				(void) fprintf(stderr,
-					"%s: no txt ptr space, PID %d\n",
-					Pn, Lp->pid);
-				Exit(1);
-			}
-			Nv = 10;
-		} else if (n >= Nv) {
-			Nv += 10;
-			if (!(Vp = (KA_T *)realloc((MALLOC_P *)Vp,
-			     (MALLOC_S)(Nv * sizeof(struct vnode *)))))
-			{
-				(void) fprintf(stderr,
-					"%s: no more txt ptr space, PID %d\n",
-					Pn, Lp->pid);
-				Exit(1);
-			}
+	    if (!Vp) {
+		if (!(Vp = (KA_T *)malloc((MALLOC_S)
+					  (sizeof(struct vnode *) * 10))))
+		{
+		    (void) fprintf(stderr, "%s: no txt ptr space, PID %d\n",
+			Pn, Lp->pid);
+		    Error();
 		}
-		Vp[n++] = (KA_T)vmp.vs_vp;
+		Nv = 10;
+	    } else if (n >= Nv) {
+		Nv += 10;
+		if (!(Vp = (KA_T *)realloc((MALLOC_P *)Vp,
+			       (MALLOC_S)(Nv * sizeof(struct vnode *)))))
+		{
+		    (void) fprintf(stderr,
+			"%s: no more txt ptr space, PID %d\n", Pn, Lp->pid);
+		    Error();
+		}
+	    }
+	    Vp[n++] = (KA_T)vmp.vs_vp;
 	}
 }
 
@@ -603,7 +660,7 @@ read_proc()
 		if ((apax = get_Nl_value("aproc", Drive_Nl, &apav)) < 0) {
 		    (void) fprintf(stderr,
 			"%s: can't get process table address pointer\n", Pn);
-		    Exit(1);
+		    Error();
 		}
 	    }
 	    if (kread((KA_T)apav, (char *)&Kp, sizeof(Kp))) {
@@ -624,13 +681,13 @@ read_proc()
 		    {
 			(void) fprintf(stderr, "%s: no proc table space\n",
 			    Pn);
-			Exit(1);
+			Error();
 		    }
 		    if (!(Pa = (KA_T *)malloc((MALLOC_S)(sz * sizeof(KA_T)))))
 		    {
 			(void) fprintf(stderr, "%s: no proc pointer space\n",
 			    Pn);
-			Exit(1);
+			Error();
 		    }
 		}
 	/*
@@ -658,7 +715,7 @@ read_proc()
 					(void) fprintf(stderr,
 						"%s: no more (%d) proc space\n",
 						Pn, sz);
-					Exit(1);
+					Error();
 				}
 				if (!(Pa = (KA_T *)realloc((MALLOC_P *)Pa,
 					(MALLOC_S)(sizeof(KA_T) * sz))))
@@ -666,7 +723,7 @@ read_proc()
 					(void) fprintf(stderr,
 					    "%s: no more (%d) proc ptr space\n",
 					    Pn, sz);
-					Exit(1);
+					Error();
 				}
 			}
 		}
@@ -681,7 +738,7 @@ read_proc()
  */
 	if (try >= PROCTRYLM) {
 		(void) fprintf(stderr, "%s: can't read proc table\n", Pn);
-		Exit(1);
+		Error();
 	}
 	if (Np < sz && !RptTm) {
 
@@ -695,7 +752,7 @@ read_proc()
 			(void) fprintf(stderr,
 				"%s: can't reduce proc table to %d\n",
 				Pn, Np);
-			Exit(1);
+			Error();
 		}
 		if (!(Pa = (KA_T *)realloc((MALLOC_P *)Pa,
 			  (MALLOC_S)(sizeof(KA_T) * Np))))
@@ -703,7 +760,7 @@ read_proc()
 			(void) fprintf(stderr,
 				"%s: can't reduce proc ptrs to %d\n",
 				Pn, Np);
-			Exit(1);
+			Error();
 		}
 	}
 }
